@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-from __future__ import print_function
 import sys
 import os
 from copy import copy, deepcopy
@@ -8,7 +6,6 @@ from .getch import getch
 
 import numpy as nm
 import scipy.sparse as sp
-import six
 
 real_types = [nm.float64]
 complex_types = [nm.complex128]
@@ -21,12 +18,6 @@ sfepy_config_dir = os.path.expanduser('~/.sfepy')
 if not os.path.exists(sfepy_config_dir):
     os.makedirs(sfepy_config_dir)
 
-if sys.version_info[0] < 3:
-    PY3 = False
-    basestr = basestring
-else:
-    PY3 = True
-    basestr = str
 
 def get_debug():
     """
@@ -152,12 +143,8 @@ def import_file(filename, package_name=None, can_reload=True):
         mod = __import__(name)
 
     if (name in sys.modules) and can_reload:
-        if PY3:
-            import importlib
-            importlib.reload(mod)
-
-        else:
-            reload(mod)
+        import importlib
+        importlib.reload(mod)
 
     if remove_path:
         sys.path.remove(path)
@@ -272,7 +259,7 @@ def spause(msg=None):
 
 ##
 # 02.01.2005
-class Struct(object):
+class Struct:
     # 03.10.2005, c
     # 26.10.2005
     def __init__(self, **kwargs):
@@ -399,7 +386,7 @@ class Struct(object):
         attribute and its counterpart in other are both Structs - these are
         merged then."""
         new = copy(self)
-        for key, val in six.iteritems(other.__dict__):
+        for key, val in other.__dict__.items():
             if hasattr(new, key):
                 sval = getattr(self, key)
                 if issubclass(sval.__class__, Struct) and \
@@ -417,7 +404,7 @@ class Struct(object):
         """Merge Structs in place. Attributes of self are left unchanged
         unless an attribute and its counterpart in other are both Structs -
         these are merged then."""
-        for key, val in six.iteritems(other.__dict__):
+        for key, val in other.__dict__.items():
             if hasattr(self, key):
                 sval = getattr(self, key)
                 if issubclass(sval.__class__, Struct) and \
@@ -436,7 +423,7 @@ class Struct(object):
     # 08.03.2005, c
     def str_all(self):
         ss = "%s\n" % self.__class__
-        for key, val in six.iteritems(self.__dict__):
+        for key, val in self.__dict__.items():
             if issubclass(self.__dict__[key].__class__, Struct):
                 ss += "  %s:\n" % key
                 aux = "\n" + self.__dict__[key].str_all()
@@ -538,7 +525,7 @@ class Container(Struct):
 
     def __setitem__(self, ii, obj):
         try:
-            if isinstance(ii, basestr):
+            if isinstance(ii, str):
                 if ii in self.names:
                     ii = self.names.index(ii)
                 else:
@@ -560,7 +547,7 @@ class Container(Struct):
 
     def __getitem__(self, ii):
         try:
-            if isinstance(ii, basestr):
+            if isinstance(ii, str):
                 ii = self.names.index(ii)
             elif not isinstance(ii, int):
                 raise ValueError('bad index type! (%s)' % type(ii))
@@ -667,7 +654,7 @@ class Container(Struct):
                 return True
             else:
                 return False
-        elif isinstance(ii, basestr):
+        elif isinstance(ii, str):
             try:
                 self.names.index(ii)
                 return True
@@ -722,7 +709,7 @@ class OneTypeList(list):
     def __getitem__(self, ii):
         if isinstance(ii, int):
             return list.__getitem__(self, ii)
-        elif isinstance(ii, basestr):
+        elif isinstance(ii, str):
             ir = self.find(ii, ret_indx=True)
             if ir:
                 return list.__getitem__(self, ir[0])
@@ -825,7 +812,7 @@ class Output(Struct):
             Append to an existing file instead of overwriting it. Use with
             `filename`.
         """
-        if not isinstance(filename, basestr):
+        if not isinstance(filename, str):
             # filename is a file descriptor.
             append = True
 
@@ -847,7 +834,7 @@ class Output(Struct):
                 self.level += 1
 
         def print_to_file(filename, msg):
-            if isinstance(filename, basestr):
+            if isinstance(filename, str):
                 fd = open(filename, 'a')
 
             else:
@@ -855,7 +842,7 @@ class Output(Struct):
 
             print(self._prefix + ('  ' * self.level) + msg, file=fd)
 
-            if isinstance(filename, basestr):
+            if isinstance(filename, str):
                 fd.close()
 
             else:
@@ -888,7 +875,7 @@ class Output(Struct):
                 self.level += 1
 
         def reset_file(filename):
-            if isinstance(filename, basestr):
+            if isinstance(filename, str):
                 output_dir = os.path.dirname(filename)
                 if output_dir and not os.path.exists(output_dir):
                     os.makedirs(output_dir)
@@ -927,7 +914,7 @@ class Output(Struct):
         return self.output_function
 
     def set_output_prefix(self, prefix):
-        assert_(isinstance(prefix, basestr))
+        assert_(isinstance(prefix, str))
         if len(prefix) > 0:
             prefix += ' '
         self._prefix = prefix
@@ -959,7 +946,7 @@ def print_structs(objs):
     """Print Struct instances in a container, works recursively. Debugging
     utility function."""
     if isinstance(objs, dict):
-        for key, vals in six.iteritems(objs):
+        for key, vals in objs.items():
             print(key)
             print_structs(vals)
     elif isinstance(objs, list):
@@ -969,7 +956,7 @@ def print_structs(objs):
         print(objs)
 
 def iter_dict_of_lists(dol, return_keys=False):
-    for key, vals in six.iteritems(dol):
+    for key, vals in dol.items():
         for ii, val in enumerate(vals):
             if return_keys:
                 yield key, ii, val
@@ -1006,7 +993,7 @@ def dict_to_struct(*args, **kwargs):
             else:
                 aux = {}
 
-            for key, val in six.iteritems(arg):
+            for key, val in arg.items():
                 if type(val) == dict:
                     try:
                         flag[level + 1]
@@ -1041,15 +1028,13 @@ def structify(obj):
             out.__dict__[key] = structify(val)
     return out
 
+# deprecated
 def is_string(var):
-    return isinstance(var, basestr)
+    return isinstance(var, str)
 
+# deprecated
 def is_integer(var):
-    if PY3:
-        return isinstance(var, int)
-
-    else:
-        return isinstance(var, (int, long))
+    return isinstance(var, int)
 
 ##
 # 23.01.2006, c
@@ -1058,7 +1043,7 @@ def is_sequence(var):
         from collections.abc import Sequence
     except ImportError:
         from collections import Sequence
-    if isinstance(var, basestr):
+    if isinstance(var, str):
         return False
     return isinstance(var, Sequence)
 
@@ -1075,10 +1060,7 @@ def insert_static_method(cls, function):
 ##
 # 23.10.2007, c
 def insert_method(instance, function):
-    if PY3:
-        meth = MethodType(function, instance)
-    else:
-        meth = MethodType(function, instance, type(instance))
+    meth = MethodType(function, instance)
     setattr(instance, function.__name__, meth)
 
 def use_method_with_name(instance, method, new_name):
@@ -1217,7 +1199,7 @@ def edit_tuple_strings(str_tuple, old, new, recur=False):
     """
     new_tuple = []
     for item in str_tuple:
-        if isinstance(item, basestr):
+        if isinstance(item, str):
             item = item.replace(old, new)
 
         elif recur and isinstance(item, tuple):
@@ -1249,10 +1231,10 @@ def edit_dict_strings(str_dict, old, new, recur=False):
     new_dict : dict
         The dictionary with edited strings.
     """
-    if isinstance(old, basestr):
+    if isinstance(old, str):
         new_dict = {}
-        for key, val in six.iteritems(str_dict):
-            if isinstance(val, basestr):
+        for key, val in str_dict.items():
+            if isinstance(val, str):
                 new_dict[key] = val.replace(old, new)
 
             elif isinstance(val, tuple):
@@ -1293,7 +1275,7 @@ def invert_dict(d, is_val_tuple=False, unique=True):
     """
     di = {}
 
-    for key, val in six.iteritems(d):
+    for key, val in d.items():
         if unique:
             if is_val_tuple:
                 for v in val:
@@ -1318,7 +1300,7 @@ def remap_dict(d, map):
     Utility function to remap state dict keys according to var_map.
     """
     out = {}
-    for new_key, key in six.iteritems(map):
+    for new_key, key in map.items():
         out[new_key] = d[key]
 
     return out
@@ -1339,7 +1321,7 @@ def dict_from_keys_init(keys, seq_class=None):
 ##
 # 16.10.2006, c
 def dict_extend(d1, d2):
-    for key, val in six.iteritems(d1):
+    for key, val in d1.items():
         val.extend(d2[key])
 
 def get_subdict(adict, keys):
@@ -1349,7 +1331,7 @@ def get_subdict(adict, keys):
     return dict((key, adict[key]) for key in keys if key in adict)
 
 def set_defaults(dict_, defaults):
-    for key, val in six.iteritems(defaults):
+    for key, val in defaults.items():
         dict_.setdefault(key, val)
 
 ##
@@ -1413,7 +1395,7 @@ def check_names(names1, names2, msg):
 # c: 27.02.2008, r: 27.02.2008
 def select_by_names(objs_all, names, replace=None, simple=True):
     objs = {}
-    for key, val in six.iteritems(objs_all):
+    for key, val in objs_all.items():
         if val.name in names:
             if replace is None:
                 objs[key] = val
@@ -1451,7 +1433,7 @@ def dict_to_array(adict):
     aux = nm.asarray(adict[ik[0]])
     out = nm.empty((ik.max() + 1,) + aux.shape, dtype=aux.dtype)
     out.fill(-1)
-    for key, val in six.iteritems(adict):
+    for key, val in adict.items():
         out[key] = val
 
     return out
