@@ -7,15 +7,13 @@ Notes
 Short syntax: key is suffixed with '__<number>' to prevent collisions with long
 syntax keys -> both cases can be used in a single input.
 """
-from __future__ import absolute_import
 import re
 import numpy as nm
 
 from sfepy.base.base import (Struct, IndexedStruct, dict_to_struct,
                              output, copy, update_dict_recursively,
-                             import_file, assert_, get_default, basestr)
+                             import_file, assert_, get_default)
 from sfepy.base.parse_conf import create_bnf
-import six
 
 _required = ['filename_mesh|filename_domain', 'field_[0-9]+|fields',
             # TODO originaly EBC were required to be specified but in some examples
@@ -48,7 +46,7 @@ def tuple_to_conf(name, vals, order):
 
 def transform_variables(adict):
     d2 = {}
-    for ii, (key, conf) in enumerate(six.iteritems(adict)):
+    for ii, (key, conf) in enumerate(adict.items()):
         if isinstance(conf, tuple):
             c2 = tuple_to_conf(key, conf, ['kind', 'field'])
             if len(conf) >= 3:
@@ -58,7 +56,7 @@ def transform_variables(adict):
                 elif kind == 'test':
                     c2.dual = conf[2]
                 elif kind == 'parameter':
-                    if isinstance(conf[2], basestr) or (conf[2] is None):
+                    if isinstance(conf[2], str) or (conf[2] is None):
                         c2.like = conf[2]
                     else:
                         c2.like = None
@@ -73,7 +71,7 @@ def transform_variables(adict):
 
 def transform_conditions(adict, prefix):
     d2 = {}
-    for ii, (key, conf) in enumerate(six.iteritems(adict)):
+    for ii, (key, conf) in enumerate(adict.items()):
         if isinstance(conf, tuple):
             if len(conf) == 2:
                 c2 = tuple_to_conf(key, conf, ['region', 'dofs'])
@@ -100,7 +98,7 @@ def transform_ics(adict):
 
 def transform_lcbcs(adict):
     d2 = {}
-    for ii, (key, conf) in enumerate(six.iteritems(adict)):
+    for ii, (key, conf) in enumerate(adict.items()):
         if isinstance(conf, tuple):
             if len(conf) >= 4:
                 if isinstance(conf[1], dict):
@@ -130,7 +128,7 @@ def transform_lcbcs(adict):
 
 def transform_epbcs(adict, prefix="epbc"):
     d2 = {}
-    for ii, (key, conf) in enumerate(six.iteritems(adict)):
+    for ii, (key, conf) in enumerate(adict.items()):
         if isinstance(conf, tuple):
             if len(conf) == 3:
                 c2 = tuple_to_conf(key, conf, ['region', 'dofs', 'match'])
@@ -150,8 +148,8 @@ def transform_dgepbcs(adict):
 
 def transform_regions(adict):
     d2 = {}
-    for ii, (key, conf) in enumerate(six.iteritems(adict)):
-        if isinstance(conf, basestr):
+    for ii, (key, conf) in enumerate(adict.items()):
+        if isinstance(conf, str):
             c2 = Struct(name=key, select=conf)
             d2['region_%s__%d' % (c2.name, ii)] = c2
         elif isinstance(conf, tuple):
@@ -169,7 +167,7 @@ def transform_regions(adict):
 
 def transform_integrals(adict):
     d2 = {}
-    for ii, (key, conf) in enumerate(six.iteritems(adict)):
+    for ii, (key, conf) in enumerate(adict.items()):
         if isinstance(conf, int):
             c2 = Struct(name=key, order=conf)
             d2['integral_%s__%d' % (c2.name, ii)] = c2
@@ -193,7 +191,7 @@ def transform_integrals(adict):
 def transform_fields(adict):
     dtypes = {'real' : nm.float64, 'complex' : nm.complex128}
     d2 = {}
-    for ii, (key, conf) in enumerate(six.iteritems(adict)):
+    for ii, (key, conf) in enumerate(adict.items()):
         if isinstance(conf, tuple):
             c2 = tuple_to_conf(key, conf,
                                ['dtype', 'shape', 'region', 'approx_order',
@@ -211,8 +209,8 @@ def transform_fields(adict):
 
 def transform_materials(adict):
     d2 = {}
-    for ii, (key, conf) in enumerate(six.iteritems(adict)):
-        if isinstance(conf, basestr):
+    for ii, (key, conf) in enumerate(adict.items()):
+        if isinstance(conf, str):
             c2 = Struct(name=key, function=conf)
             d2['material_%s__%d' % (c2.name, ii)] = c2
 
@@ -231,10 +229,10 @@ def transform_materials(adict):
 
 def transform_solvers(adict):
     d2 = {}
-    for ii, (key, conf) in enumerate(six.iteritems(adict)):
+    for ii, (key, conf) in enumerate(adict.items()):
         if isinstance(conf, tuple):
             c2 = tuple_to_conf(key, conf, ['kind','params'])
-            for param, val in six.iteritems(c2.params):
+            for param, val in c2.params.items():
                 setattr(c2, param, val)
             delattr(c2, 'params')
             d2['solvers_%s__%d' % (c2.name, ii)] = c2
@@ -245,7 +243,7 @@ def transform_solvers(adict):
 
 def transform_functions(adict):
     d2 = {}
-    for ii, (key, conf) in enumerate(six.iteritems(adict)):
+    for ii, (key, conf) in enumerate(adict.items()):
         if isinstance(conf, tuple):
             c2 = tuple_to_conf(key, conf, ['function'])
             d2['function_%s__%d' % (c2.name, ii)] = c2
@@ -440,7 +438,7 @@ class ProblemConf(Struct):
 
         self.transform_input_trivial()
         self._raw = {}
-        for key, val in six.iteritems(define_dict):
+        for key, val in define_dict.items():
             if isinstance(val, dict):
                 self._raw[key] = copy(val)
 
@@ -515,7 +513,7 @@ class ProblemConf(Struct):
 
     def transform_input(self):
         keys = list(self.__dict__.keys())
-        for key, transform in six.iteritems(transforms):
+        for key, transform in transforms.items():
             if not key in keys: continue
             self.__dict__[key] = transform(self.__dict__[key])
 
@@ -531,7 +529,7 @@ class ProblemConf(Struct):
         by `key`.
         """
         val = getattr(self, key)
-        for item in six.itervalues(val):
+        for item in val.values():
             if item.name == item_name:
                 return item
 
