@@ -4,8 +4,6 @@ getch()-like unbuffered character reading from stdin on both Windows and Unix
 _Getch classes inspired by Danny Yoo, iskeydown() based on code by Zachary
 Pincus.
 """
-from __future__ import absolute_import
-from __future__ import print_function
 import os, sys
 
 class _Getch:
@@ -28,7 +26,7 @@ class _GetchWindows:
 
     def __call__(self):
         msvcrt = self.msvcrt
-        
+
         c = msvcrt.getch()
         if c == '\x00' or c == '\xE0':    #functions keys
             msvcrt.getch()
@@ -40,7 +38,7 @@ class _GetchWindows:
 
 class _GetchUnix:
     def __init__(self):
-        import tty, sys, select
+        import tty, select
         self.mods = (tty, sys, select)
 
     def __call__(self):
@@ -70,7 +68,7 @@ class _GetchUnix:
 
 class _GetchDefault:
     def __call__(self):
-        return raw_input()[0]
+        return input()[0]
 
     def iskeydown(self):
         raise NotImplementedError
@@ -79,7 +77,7 @@ getch = _Getch()
 
 if __name__ == '__main__':
     from .base import pause, spause
-    
+
     pause('press a key anytime the script stops!')
     pause()
     spause('last time...')
