@@ -1,15 +1,12 @@
-from __future__ import absolute_import
 import time
 import os
 import atexit
-import six
-from six.moves import range
 
 try:
-    from multiprocessing import Process, Pipe
+    import multiprocessing as mp
 
 except ImportError:
-    Process = None
+    mp = None
 
 import numpy as nm
 
@@ -82,6 +79,7 @@ def read_log(filename):
 
     log = {}
     info = {}
+    name2key = {}
 
     fd = open(filename, 'r')
 
@@ -90,6 +88,7 @@ def read_log(filename):
             ls = line.split(':')
             if ls[0] == '# groups':
                 n_gr = int(ls[1])
+                offset = 0
                 for ig in range(n_gr):
                     next(fd)
                     line_info = next(fd)
@@ -97,22 +96,34 @@ def read_log(filename):
 
                     line_names = next(fd)
                     names = line_names.split(':')[1]
+                    names = [name.strip().strip('"')
+                             for name in names.split(',')]
+                    if len(names[0]) == 0:
+                        names = []
 
                     line_plot_kwargs = next(fd)
+                    aux = line_plot_kwargs[19:].strip().strip('"')
+                    plot_kwargs = eval(aux + ',') if len(aux) else ({},)
 
                     info[ig] = (xlabel.split(':')[1].strip().strip('"'),
                                 ylabel.split(':')[1].strip().strip('"'),
                                 yscales.split(':')[1].strip().strip('"'),
-                                [name.strip().strip('"')
-                                 for name in names.split(',')],
-                                eval(line_plot_kwargs[19:].strip().strip('"')
-                                     + ','))
+                                names,
+                                plot_kwargs)
 
+                    name2key.update({name : ik + offset
+                                     for ik, name in enumerate(info[ig][3])})
+                    offset += len(info[ig][3])
             continue
 
         ls = line.split(':')
 
-        key = int(ls[0])
+        try:
+            key = int(ls[0])
+
+        except ValueError:
+            key = name2key[ls[0]] # Old style log.
+
         xs, ys, vlines = log.setdefault(key, ([], [], []))
 
         if (len(ls) == 2) and len(log[key][0]):
@@ -131,7 +142,7 @@ def read_log(filename):
 
     fd.close()
 
-    for key, (xs, ys, vlines) in six.iteritems(log):
+    for key, (xs, ys, vlines) in log.items():
         log[key] = (nm.array(xs), nm.array(ys), nm.array(vlines))
 
     return log, info
@@ -387,7 +398,7 @@ class Log(Struct):
                            yscales[ig], xlabels[ig], ylabels[ig],
                            formats[ig])
 
-        self.can_plot = (mpl is not None) and (Process is not None)
+        self.can_plot = (mpl is not None) and (mp is not None)
 
         if log_filename is not None:
             self.output = Output('', filename=log_filename)
@@ -518,16 +529,18 @@ class Log(Struct):
 
                 self.__class__.count += 1
 
-                self.plot_pipe, plotter_pipe = Pipe()
+                ctx = mp.get_context('spawn')
+
+                self.plot_pipe, plotter_pipe = ctx.Pipe()
                 self.plotter = LogPlotter(self.aggregate, self.sleep)
-                self.plot_process = Process(target=self.plotter,
-                                            args=(plotter_pipe,
-                                                  self.get_log_name(),
-                                                  self.data_names,
-                                                  self.yscales,
-                                                  self.xlabels,
-                                                  self.ylabels,
-                                                  self.plot_kwargs))
+                self.plot_process = ctx.Process(target=self.plotter,
+                                                args=(plotter_pipe,
+                                                      self.get_log_name(),
+                                                      self.data_names,
+                                                      self.yscales,
+                                                      self.xlabels,
+                                                      self.ylabels,
+                                                      self.plot_kwargs))
                 self.plot_process.daemon = True
                 self.plot_process.start()
 
@@ -546,6 +559,8 @@ class Log(Struct):
             self.n_calls = 0
             output('terminated')
 
+        atexit.unregister(self.terminate)
+
     def plot_data(self, igs):
         send = self.plot_pipe.send
 
