@@ -1,4 +1,4 @@
-from base import Struct
+from .base import Struct
 import os.path as op
 
 ##
@@ -41,7 +41,7 @@ class Reader( Struct ):
         execfile( filename, {}, aux )
 
         obj = obj_class()
-        for key, val in aux.iteritems():
+        for key, val in aux.items():
             obj.__dict__[key] = val
 
         return obj
