@@ -1,19 +1,15 @@
 """
 Functions for resolving dependencies.
 """
-from __future__ import absolute_import
 import itertools as it
 
-from sfepy.base.base import basestr
-import six
-from six.moves import range
 
 def get_nums(deps):
     """
     Get number of prerequisite names for each name in dependencies.
     """
     nums = {}
-    for key, val in six.iteritems(deps):
+    for key, val in deps.items():
         nums[key] = len(val)
 
     return nums
@@ -35,9 +31,9 @@ def remove_known(deps, known):
     """
     Remove known names from dependencies.
     """
-    if isinstance(known, basestr):
+    if isinstance(known, str):
         out = {}
-        for key, val in six.iteritems(deps):
+        for key, val in deps.items():
             if key == known: continue
             out[key] = [ii for ii in val if ii != known]
 
