@@ -7,7 +7,7 @@ Helper functions for the equation mapping.
 import numpy as nm
 import scipy.sparse as sp
 
-from sfepy.base.base import assert_, Struct, basestr
+from sfepy.base.base import assert_, Struct
 from sfepy.discrete.functions import Function
 from sfepy.discrete.conditions import get_condition_value, EssentialBC, \
     PeriodicBC, DGPeriodicBC, DGEssentialBC
@@ -64,11 +64,11 @@ def group_chains(chain_list):
     chains = []
     while len(chain_list):
         chain = set(chain_list.pop(0))
-        ## print ':', chain
+        ## print(':', chain)
         ii = 0
         while ii < len(chain_list):
             c1 = sorted(chain_list[ii])
-            ## print '--', ii, c1, chain
+            ## print('--', ii, c1, chain)
             is0 = c1[0] in chain
             is1 = c1[1] in chain
 
@@ -80,17 +80,17 @@ def group_chains(chain_list):
                 ii = 0
             else:
                 ii += 1
-            ## print ii, chain, chain_list
-        ## print '->', chain
-        ## print chain_list
+            ## print(ii, chain, chain_list)
+        ## print('->', chain)
+        ## print(chain_list)
 
         chains.append(list(chain))
 
-    ## print 'EPBC chain groups:', chains
+    ## print('EPBC chain groups:', chains)
     aux = {}
     for chain in chains:
         aux.setdefault(len(chain), [0])[0] += 1
-    ## print 'EPBC chain counts:', aux
+    ## print('EPBC chain counts:', aux)
 
     return chains
 
@@ -106,19 +106,25 @@ class DofInfo(Struct):
         self.n_var = 0
         self.var_names = []
         self.n_dof = {}
-        self.ptr = [0]
+        self.n_dof_total = 0
         self.indx = {}
         self.details = {}
+        self.shared_dofs = {}
 
-    def _update_after_append(self, name):
-        self.ptr.append(self.ptr[-1] + self.n_dof[name])
-
-        ii = self.n_var
-        self.indx[name] = slice(int(self.ptr[ii]), int(self.ptr[ii+1]))
+    def _update_after_append(self, name, shared=None):
+        if shared is None:
+            indx0 = self.n_dof_total
+            n_dof = self.n_dof[name]
+            self.n_dof_total += n_dof
+            self.indx[name] = slice(indx0, indx0 + n_dof)
+        else:
+            self.indx[name] = self.indx[shared]
+            self.n_dof[name] = self.n_dof[shared]
+            self.shared_dofs[name] = shared
 
         self.n_var += 1
 
-    def append_variable(self, var, active=False):
+    def append_variable(self, var, active=False, shared=None):
         """
         Append DOFs of the given variable.
 
@@ -136,7 +142,7 @@ class DofInfo(Struct):
         self.var_names.append(name)
 
         self.n_dof[name], self.details[name] = var.get_dof_info(active=active)
-        self._update_after_append(name)
+        self._update_after_append(name, shared=shared)
 
     def append_raw(self, name, n_dof):
         """
@@ -157,29 +163,6 @@ class DofInfo(Struct):
         self.n_dof[name], self.details[name] = n_dof, None
         self._update_after_append(name)
 
-    def update(self, name, n_dof):
-        """
-        Set the number of DOFs of the given variable.
-
-        Parameters
-        ----------
-        name : str
-            The name of variable the DOFs correspond to.
-        n_dof : int
-            The number of DOFs.
-        """
-        if not name in self.var_names:
-            raise ValueError('variable %s is not present!' % name)
-
-        ii = self.var_names.index(name)
-        delta = n_dof - self.n_dof[name]
-
-        self.n_dof[name] = n_dof
-
-        for iv, nn in enumerate(self.var_names[ii:]):
-            self.ptr[ii+iv+1] += delta
-            self.indx[nn] = slice(self.ptr[ii+iv], self.ptr[ii+iv+1])
-
     def get_info(self, var_name):
         """
         Return information on DOFs of the given variable.
@@ -193,7 +176,8 @@ class DofInfo(Struct):
                       var_name=var_name,
                       n_dof=self.n_dof[var_name],
                       indx=self.indx[var_name],
-                      details=self.details[var_name])
+                      details=self.details[var_name],
+                      shared_dofs_with=self.shared_dofs.get(var_name, None))
 
     def get_subset_info(self, var_names):
         """
@@ -218,7 +202,7 @@ class DofInfo(Struct):
         """
         Return the total number of DOFs of all state variables.
         """
-        return self.ptr[-1]
+        return self.n_dof_total
 
 def is_active_bc(bc, ts=None, functions=None):
     """
@@ -243,7 +227,7 @@ def is_active_bc(bc, ts=None, functions=None):
             active = False
 
     else:
-        if isinstance(bc.times, basestr):
+        if isinstance(bc.times, str):
             if functions is not None:
                 fun = functions[bc.times]
 
@@ -355,19 +339,26 @@ class EquationMap(Struct):
             if not is_active_bc(bc, ts=ts, functions=functions):
                 continue
 
-            active_bcs.add(bc.key)
             if isinstance(bc, DGEssentialBC):
                 ntype = "DGEBC"
                 region = bc.region
+                sig = (bc.key, bc.name, tuple(bc.dofs[0]), bc.region.name)
             elif isinstance(bc, DGPeriodicBC):
                 ntype = "DGEPBC"
                 region = bc.regions[0]
+                sig = (bc.key, bc.name, tuple(bc.dofs[0]), tuple(bc.dofs[1]),
+                       bc.regions[0].name, bc.regions[1].name)
             elif isinstance(bc, EssentialBC):
                 ntype = 'EBC'
                 region = bc.region
+                sig = (bc.key, bc.name, tuple(bc.dofs[0]), bc.region.name)
             elif isinstance(bc, PeriodicBC):
                 ntype = 'EPBC'
                 region = bc.regions[0]
+                sig = (bc.key, bc.name, tuple(bc.dofs[0]), tuple(bc.dofs[1]),
+                       bc.regions[0].name, bc.regions[1].name)
+
+            active_bcs.add(sig)
 
             if warn:
                 clean_msg = ('warning: ignoring nonexistent %s node (%s) in '
@@ -436,8 +427,8 @@ class EquationMap(Struct):
                 if nslave[0] == -1:
                     nslave = nslave[1:]
 
-                ## print nmaster + 1
-                ## print nslave + 1
+                ## print(nmaster + 1)
+                ## print(nslave + 1)
                 if nmaster.shape != nslave.shape:
                     msg = 'EPBC list lengths do not match!\n(%s,\n %s)' %\
                           (nmaster, nslave)
@@ -456,8 +447,8 @@ class EquationMap(Struct):
                 else:
                     i1, i2 = fun
 
-                ## print nm.c_[mcoor[i1], scoor[i2]]
-                ## print nm.c_[nmaster[i1], nslave[i2]] + 1
+                ## print(nm.c_[mcoor[i1], scoor[i2]])
+                ## print(nm.c_[nmaster[i1], nslave[i2]] + 1)
 
                 meq = expand_nodes_to_equations(nmaster[i1], bc.dofs[0],
                                                 self.dof_names)
@@ -496,20 +487,31 @@ class EquationMap(Struct):
         chains = group_chains(chains)
         resolve_chains(master_slave, chains)
 
-        ii = nm.argwhere(eq_ebc == 1)
-        self.eq_ebc = nm.atleast_1d(ii.squeeze())
-        self.val_ebc = nm.atleast_1d(val_ebc[ii].squeeze())
-        # add axis in case we squeezed too hard
-        self.master = nm.atleast_1d(nm.argwhere(master_slave > 0).squeeze())
+        self.master = nm.nonzero(master_slave > 0)[0]
         self.slave = master_slave[self.master] - 1
 
+        # Propagate EBCs via PBCs.
+        mask = eq_ebc[self.master] > 0
+        im0 = self.master[mask]
+        im1 = self.slave[mask]
+        mask = eq_ebc[self.slave] > 0
+        is0 = self.slave[mask]
+        is1 = self.master[mask]
+        val_ebc[im1] = val_ebc[im0]
+        eq_ebc[im1] = eq_ebc[im0]
+        val_ebc[is1] = val_ebc[is0]
+        eq_ebc[is1] = eq_ebc[is0]
+
+        self.eq_ebc = nm.nonzero(eq_ebc > 0)[0]
+        self.val_ebc = val_ebc[self.eq_ebc]
         assert_((self.eq_ebc.shape == self.val_ebc.shape))
+
         self.eq[self.eq_ebc] = -2
         self.eq[self.master] = -1
 
         self._mark_unused(field)
 
-        self.eqi = nm.compress(self.eq >= 0, self.eq)
+        self.eqi = self.eq[self.eq >= 0]
         self.eq[self.eqi] = nm.arange(self.eqi.shape[0], dtype=nm.int32)
         self.eq[self.master] = self.eq[self.slave]
         self.n_eq = self.eqi.shape[0]
