@@ -910,13 +910,13 @@ int32 mesh_get_centroids(Mesh *mesh, float64 *ccoors, int32 dim)
   return(RET_OK);
 }
 
-inline float64 _det3x3(float64 j[9])
+static inline float64 _det3x3(float64 j[9])
 {
   return (j[0]*j[4]*j[8] + j[3]*j[7]*j[2] + j[1]*j[5]*j[6]
           - j[2]*j[4]*j[6] - j[5]*j[7]*j[0] - j[1]*j[3]*j[8]);
 }
 
-inline float64 _tri_area(float64 *coors, uint32 *indices, uint32 nc)
+static inline float64 _tri_area(float64 *coors, uint32 *indices, uint32 nc)
 {
 #define VS(ic, id) (coors[nc*indices[ic] + id])
 
@@ -945,8 +945,8 @@ inline float64 _tri_area(float64 *coors, uint32 *indices, uint32 nc)
 #undef VS
 }
 
-inline float64 _aux_hex(float64 *coors, uint32 *indices, uint32 nc,
-                        int32 h, int32 i, int32 j, int32 k)
+static inline float64 _aux_hex(float64 *coors, uint32 *indices, uint32 nc,
+                               int32 h, int32 i, int32 j, int32 k)
 {
 #define VS(ic, id) (coors[nc*indices[ic] + id])
 
@@ -1046,7 +1046,20 @@ int32 mesh_get_volumes(Mesh *mesh, float64 *volumes, int32 dim)
           gtr_dot_v3(ptr, v2, ndir, 3);
           ptr[0] /= 6.0;
         }
+      } else if (entity_vertices->num == 6) { // Wedge cells
+          for (id = 0; id < nc; id++) {
+            vv0 = VS(0, id);
+            vv1 = VS(1, id);
+            vv2 = VS(2, id);
+            vv3 = VS(3, id);
 
+            v0[id] = vv1 - vv0;
+            v1[id] = vv2 - vv0;
+            v2[id] = vv3 - vv2;
+          }
+          gtr_cross_product(ndir, v0, v1);
+          gtr_dot_v3(ptr, v2, ndir, 3);
+          ptr[0] /= 2.0;
       } else { // Hexahedral cells with trilinear interpolation.
         // See https://math.stackexchange.com/questions/1628540/what-is-the-enclosed-volume-of-an-irregular-cube-given-the-x-y-z-coordinates-of
         // Uses 0 1 3 2 4 5 7 6 ordering w.r.t. sfepy.
