@@ -1,11 +1,8 @@
-from __future__ import absolute_import
 
 import numpy as nm
 
-from sfepy.base.base import output, iter_dict_of_lists, Struct, basestr,\
-    assert_
+from sfepy.base.base import output, iter_dict_of_lists, Struct, assert_
 from sfepy.base.timing import Timer
-import six
 from sfepy.mechanics.tensors import get_cauchy_strain
 
 
@@ -20,7 +17,7 @@ def parse_approx_order(approx_order):
     if approx_order is None:
         return 'iga', force_bubble, discontinuous
 
-    elif isinstance(approx_order, basestr):
+    elif isinstance(approx_order, str):
         if approx_order.startswith('iga'):
             return approx_order, force_bubble, discontinuous
 
@@ -49,14 +46,14 @@ def parse_approx_order(approx_order):
     return ao, force_bubble, discontinuous
 
 def parse_shape(shape, dim):
-    if isinstance(shape, basestr):
+    if isinstance(shape, str):
         try:
             shape = {'scalar' : (1,),
                      'vector' : (dim,)}[shape]
         except KeyError:
             raise ValueError('unsupported field shape! (%s)', shape)
 
-    elif isinstance(shape, six.integer_types):
+    elif isinstance(shape, int):
         shape = (int(shape),)
 
     return shape
@@ -73,7 +70,7 @@ def setup_extra_data(conn_info):
 
 def fields_from_conf(conf, regions):
     fields = {}
-    for key, val in six.iteritems(conf):
+    for key, val in conf.items():
         field = Field.from_conf(val, regions)
         fields[field.name] = field
 
@@ -204,7 +201,7 @@ class Field(Struct):
         import sfepy.base.multiproc as multi
 
         if multi.is_remote_dict(self.mappings0):
-            for k, v in six.iteritems(self.mappings):
+            for k, v in self.mappings.items():
                 m, _ = self.mappings[k]
                 nv = (m.bf, m.bfg, m.det, m.volume, m.normal)
                 self.mappings0[k] = nv
