@@ -1,14 +1,11 @@
-from __future__ import absolute_import
 import re
-from copy import copy
 
 import numpy as nm
 
 from sfepy.base.base import assert_, Struct
-import six
-from six.moves import range
+from sfepy.base.compat import in1d
 
-_depends = re.compile('r\.([a-zA-Z_\-0-9.]+)').findall
+_depends = re.compile(r'r\.([a-zA-Z_\-0-9.]+)').findall
 
 def get_parents(selector):
     """
@@ -25,7 +22,7 @@ def get_dependency_graph(region_defs):
     """
     graph = {}
     name_to_sort_name = {}
-    for sort_name, rdef in six.iteritems(region_defs):
+    for sort_name, rdef in region_defs.items():
         name, sel = rdef.name, rdef.select
         if name in name_to_sort_name:
             msg = 'region %s/%s already defined!' % (sort_name, name)
@@ -52,7 +49,7 @@ def sort_by_dependency(graph):
     while idone < n_nod:
 
         dep_removed = 0
-        for node, deps in six.iteritems(graph):
+        for node, deps in graph.items():
 
             if (len(deps) == 1) and not deps[0]:
                 out.append(node)
@@ -215,7 +212,7 @@ class Region(Struct):
         return obj
 
     def __init__(self, name, definition, domain, parse_def, kind='cell',
-                 parent=None):
+                 parent=None, tdim=None):
         """
         Create region instance.
 
@@ -235,8 +232,11 @@ class Region(Struct):
             'cell_only', ..., 'vertex_only'.
         parent : str, optional
             The name of the parent region.
+        tdim : int
+            The topological dimension of the cells.
         """
-        tdim = domain.shape.tdim
+        if tdim is None:
+            tdim = domain.shape.tdim
         Struct.__init__(self,
                         name=name, definition=definition,
                         domain=domain, parse_def=parse_def,
@@ -244,12 +244,16 @@ class Region(Struct):
                         tdim=tdim, kind_tdim=None,
                         entities=[None] * (tdim + 1),
                         kind=None, parent=parent, shape=None,
-                        mirror_region=None, is_empty=False)
+                        mirror_regions={}, mirror_maps={}, is_empty=False,
+                        cmesh=domain.cmesh_tdim[tdim])
         self.set_kind(kind)
 
     def set_kind(self, kind):
         if kind == self.kind: return
 
+        if self.__facet_kinds[self.tdim]['facet'] == kind:
+            kind = 'facet'
+
         self.kind = kind
         if 'facet' in kind:
             self.true_kind = self.__facet_kinds[self.tdim][kind]
@@ -260,21 +264,22 @@ class Region(Struct):
         can = [bool(ii) for ii in self.__can[self.true_kind]]
 
         self.can_vertices = can[0]
+        self.can_cells = can[3]
 
         if self.tdim == 1:
             self.can = (can[0], can[3])
-            self.can_cells = can[1]
+            self.can_edges = False
+            self.can_faces = False
 
         elif self.tdim == 2:
             self.can = (can[0], can[1], can[3])
             self.can_edges = can[1]
-            self.can_cells = can[2]
+            self.can_faces = False
 
         else:
             self.can = can
             self.can_edges = can[1]
             self.can_faces = can[2]
-            self.can_cells = can[3]
 
         for ii, ican in enumerate(self.can):
             if not ican:
@@ -441,7 +446,7 @@ class Region(Struct):
 
             return
 
-        cmesh = self.domain.cmesh
+        cmesh = self.cmesh
         if idim <= dim:
             if not (allow_lower or allow_empty):
                 msg = 'setup_from_highest() can be used only with dim < %d'
@@ -471,7 +476,7 @@ class Region(Struct):
         """
         if not self.can[dim]: return
 
-        cmesh = self.domain.cmesh
+        cmesh = self.cmesh
         cmesh.setup_connectivity(dim, 0)
         vv = self.vertices
         self.entities[dim] = cmesh.get_complete(dim, vv, 0)
@@ -483,8 +488,12 @@ class Region(Struct):
         from the kind entities.
         """
         self._access(self.kind_tdim)
-        if allow_empty and ('cell' in self.true_kind):
-            self.is_empty = self.cells.shape[0] == 0
+        is_empty = self.entities[self.kind_tdim].shape[0] == 0
+        if allow_empty:
+            self.is_empty = is_empty
+
+        elif is_empty:
+            raise ValueError('region "%s" has no entities!' % self.name)
 
         for idim in range(self.kind_tdim - 1, -1, -1):
             if self.can[idim] and self.entities[idim] is not None:
@@ -500,49 +509,49 @@ class Region(Struct):
 
     def eval_op_vertices(self, other, op):
         parse_def = _join(self.parse_def, '%sv' % op, other.parse_def)
-        tmp = self.light_copy('op', parse_def)
+        tmp = self.light_copy('op', parse_def, tdim=self.tdim)
         tmp.vertices = self.__op_to_fun[op](self.vertices, other.vertices)
 
         return tmp
 
     def eval_op_edges(self, other, op):
         parse_def = _join(self.parse_def, '%se' % op, other.parse_def)
-        tmp = self.light_copy('op', parse_def)
+        tmp = self.light_copy('op', parse_def, tdim=self.tdim)
         tmp.edges = self.__op_to_fun[op](self.edges, other.edges)
 
         return tmp
 
     def eval_op_faces(self, other, op):
         parse_def = _join(self.parse_def, '%sf' % op, other.parse_def)
-        tmp = self.light_copy('op', parse_def)
+        tmp = self.light_copy('op', parse_def, tdim=self.tdim)
         tmp.faces = self.__op_to_fun[op](self.faces, other.faces)
 
         return tmp
 
     def eval_op_facets(self, other, op):
         parse_def = _join(self.parse_def, '%ss' % op, other.parse_def)
-        tmp = self.light_copy('op', parse_def)
+        tmp = self.light_copy('op', parse_def, tdim=self.tdim)
         tmp.facets = self.__op_to_fun[op](self.facets, other.facets)
 
         return tmp
 
     def eval_op_cells(self, other, op):
         parse_def = _join(self.parse_def, '%sc' % op, other.parse_def)
-        tmp = self.light_copy('op', parse_def)
+        tmp = self.light_copy('op', parse_def, tdim=self.tdim)
         tmp.cells = self.__op_to_fun[op](self.cells, other.cells)
 
         return tmp
 
-    def light_copy(self, name, parse_def):
+    def light_copy(self, name, parse_def, tdim=None):
         return Region(name, self.definition, self.domain, parse_def,
-                      kind=self.kind)
+                      kind=self.kind, tdim=tdim)
 
     def copy(self):
         """
-        Vertices-based copy.
+        Make a copy based on the region kind.
         """
-        tmp = self.light_copy('copy', self.parse_def)
-        tmp.vertices = copy(self.vertices)
+        tmp = self.light_copy('copy', self.parse_def, tdim=self.tdim)
+        tmp.entities[self.kind_tdim] = self.get_entities(self.kind_tdim).copy()
 
         return tmp
 
@@ -558,7 +567,7 @@ class Region(Struct):
         n_cell = self.cells.shape[0]
         n_edge = self.edges.shape[0] if self.tdim > 1 else 0
         n_face = self.faces.shape[0] if self.tdim == 3 else 0
-        n_facet = self.facets.shape[0] if self.tdim > 1 else 0
+        n_facet = self.facets.shape[0]
 
         self.shape = Struct(n_vertex=n_vertex,
                             n_edge=n_edge,
@@ -588,7 +597,7 @@ class Region(Struct):
         to facets are returned if the region itself contains no cells. Obeys
         parent region, if given.
         """
-        if self.cells.shape[0] == 0:
+        if self.kind != 'cell':
             if true_cells_only:
                 msg = 'region %s has not true cells! (has kind: %s)' \
                       % (self.name, self.kind)
@@ -596,13 +605,13 @@ class Region(Struct):
 
             else:
                 # Has to be consistent with get_facet_indices()!
-                cmesh = self.domain.cmesh
+                cmesh = self.cmesh
                 cmesh.setup_connectivity(self.tdim - 1, self.tdim)
                 out = cmesh.get_incident(self.tdim, self.facets, self.tdim - 1)
 
                 if self.parent is not None:
                     pcells = self.domain.regions[self.parent].cells
-                    ip = nm.in1d(out, pcells, assume_unique=False)
+                    ip = in1d(out, pcells, assume_unique=False)
                     out = out[ip]
 
         else:
@@ -618,27 +627,18 @@ class Region(Struct):
         not allow cells. For `true_cells_only` equal to False, cells incident
         to facets are returned if the region itself contains no cells.
 
-        Notes
-        -----
-        If the number of unique values in `cells` is smaller or equal to the
-        number of cells in the region, all `cells` has to be also the region
-        cells (`self` is a superset of `cells`). The region cells are
-        considered depending on `true_cells_only`.
-
-        Otherwise, indices of all cells in `self` that are in `cells` are
-        returned.
+        Raises ValueError if all `cells` are not in the region cells.
         """
         fcells = self.get_cells(true_cells_only=true_cells_only)
 
-        if len(nm.unique(cells)) <= len(nm.unique(fcells)):
-            # self is a superset of cells.
-            ii = nm.searchsorted(fcells, cells)
-            assert_((fcells[ii] == cells).all())
+        iin = nm.isin(cells, fcells)
+        if not iin.all():
+            raise ValueError(f'some cells are not in region {self.name} cells!')
 
-        else:
-            aux = nm.searchsorted(cells, fcells)
-            ii = nm.where(nm.take(cells, aux, mode='clip') == fcells)[0]
+        iis = nm.searchsorted(fcells, cells)
+        assert_((fcells[iis[iin]] == cells[iin]).all())
 
+        ii = nm.where(iin, iis, -1)
         return ii
 
     def get_facet_indices(self):
@@ -646,7 +646,7 @@ class Region(Struct):
         Return an array (per group) of (iel, ifa) for each facet. A facet can
         be in 1 (surface) or 2 (inner) cells.
         """
-        cmesh = self.domain.cmesh
+        cmesh = self.cmesh
         cmesh.setup_connectivity(self.tdim - 1, self.tdim)
 
         facets = self.facets
@@ -655,7 +655,7 @@ class Region(Struct):
 
         if self.parent is not None:
             pcells = self.domain.regions[self.parent].cells
-            ip = nm.in1d(cells, pcells, assume_unique=False)
+            ip = in1d(cells, pcells, assume_unique=False)
             cells = cells[ip]
 
             counts = nm.diff(offs).astype(nm.int32)
@@ -668,26 +668,77 @@ class Region(Struct):
 
         return fis
 
-    def setup_mirror_region(self):
+    def setup_mirror_region(self, mirror_name=None, ret_name=False):
         """
         Find the corresponding mirror region, set up element mapping.
         """
+        from sfepy.discrete.fem.mesh import find_map
+
         regions = self.domain.regions
+        eopts = self.extra_options
 
-        for reg in regions:
-            mirror_parent = regions.find(reg.parent)
-            if mirror_parent is None: continue
-            if ((reg is not self)
-                and nm.all(self.vertices == reg.vertices)):
-                mirror_region = reg
-                break
+        if (mirror_name is None) and (eopts is not None)\
+            and ('mirror_region' in eopts):
+            mirror_name = eopts['mirror_region']
+
+        if mirror_name is not None:
+            if mirror_name in self.mirror_regions:
+                return mirror_name if ret_name else None
+
+            mreg = regions[mirror_name]
+            if self.vertices.shape[0] != mreg.vertices.shape[0]:
+                raise ValueError('%s: incompatible mirror region! (%s)'
+                    % (self.name, mreg.name))
+            coors = self.cmesh.coors
+            coors1 = coors[self.vertices, :]
+            coors2 = coors[mreg.vertices, :]
+            shift = ((nm.sum(coors2, axis=0) - nm.sum(coors1, axis=0))
+                / coors1.shape[0])
+            vmap1, vmap2 = find_map(coors1, coors2 - shift, join=False)
+            if vmap1.shape[0] != coors1.shape[0]:
+                print(coors1[vmap1])
+                print(coors2[vmap2])
+                raise ValueError('cannot match vertices!')
+
+            _ = self.cmesh.get_conn_as_graph(self.dim - 1, 0)
+            cc1 = self.cmesh.get_centroids(self.dim - 1)
+            sfacets = self.cells if self.tdim < self.dim else self.facets
+            cc2 = mreg.cmesh.get_centroids(mreg.dim - 1)
+            mfacets = mreg.cells if mreg.tdim < mreg.dim else mreg.facets
+            fmap1, fmap2 = find_map(cc1[sfacets], cc2[mfacets] - shift,
+                                    join=False)
+            if fmap1.shape[0] != sfacets.shape[0]:
+                print(cc1[sfacets][fmap1])
+                print(cc2[mfacets][fmap2])
+                raise ValueError('cannot match facets!')
+
+            mirror_map_i = nm.zeros_like(fmap1)
+            mirror_map_i[fmap1] = fmap2
+            mirror_map = nm.zeros_like(fmap1)
+            mirror_map[fmap2] = fmap1
         else:
-            raise ValueError('cannot find mirror region! (%s)' % self.name)
+            for reg in regions:
+                mirror_parent = regions.find(reg.parent)
+                if mirror_parent is None: continue
+                if ((reg is not self)
+		    and (len(self.vertices)) == len(reg.vertices)
+                    and nm.all(self.vertices == reg.vertices)):
+                    mreg = reg
+                    mirror_map = mirror_map_i = None
+                    break
+            else:
+                raise ValueError('cannot find mirror region! (%s)' % self.name)
 
-        self.mirror_region = mirror_region
+        self.mirror_regions[mreg.name] = mreg
+        self.mirror_maps[mreg.name] = mirror_map
+        mreg.mirror_regions[self.name] = self
+        mreg.mirror_maps[self.name] = mirror_map_i
 
-    def get_mirror_region(self):
-        return self.mirror_region
+        if ret_name:
+            return mreg.name
+
+    def get_mirror_region(self, name):
+        return self.mirror_regions[name]
 
     def get_n_cells(self, is_surface=False):
         """
@@ -758,7 +809,7 @@ class Region(Struct):
         """
         from scipy.sparse import csr_matrix
 
-        cmesh = self.domain.cmesh
+        cmesh = self.cmesh
 
         e_verts = cmesh.get_incident(0, self.edges, 1)
         e_verts.shape = (e_verts.shape[0] // 2, 2)
