@@ -33,12 +33,51 @@ Square: ori - iter:
 Possible couples:
 
 7, 33, 52, 63 <-> 0, 11, 30, 56
+
+_quad_ori_groups:
+
+i < j < k < l
+
+all faces are permuted to
+
+l --- k
+|     |
+|     |
+i --- j
+
+ijkl
+
+which is the same as
+
+l --- j
+|     |
+|     |
+i --- k
+
+ikjl
+
+k --- l
+|     |
+|     |
+i --- j
+
+ijlk
+
+- start at one vertex and go around clock-wise or anticlock-wise
+-> 8 groups of 3
+-> same face nodes order in
+ijkl (63), ikjl (59), ijlk (31)
+ilkj (11), iklj (15), iljk (43)
+jkli ( 7), jlki ( 3), kjli ( 6)
+kjil (56), jkil (57), ljik (48)
+lijk (52), likj (20), kijl (60)
+lkji ( 0), ljki ( 4), klji ( 1)
+klij (33), lkij (32), jlik (41)
+jilk (30), kilj (22), jikl (62)
 """
-from __future__ import print_function
-from __future__ import absolute_import
+from itertools import permutations
+
 import numpy as nm
-import six
-from six.moves import range
 
 _quad_ori_groups = {
     0 : 0,
@@ -76,8 +115,6 @@ def build_orientation_map(n_fp):
     used to sort facet vertices lexicographically. Hence `permuted_facet =
     facet[permutation]`.
     """
-    from sfepy.linalg import permutations
-
     indices = list(range(n_fp))
 
     cmps = [(i1, i2) for i2 in indices for i1 in indices[:i2]]
@@ -249,7 +286,7 @@ def get_facet_dof_permutations(n_fp, order):
     elif n_fp == 4:
         mtx = make_square_matrix(order)
         ori_map = {}
-        for key, val in six.iteritems(_quad_ori_groups):
+        for key, val in _quad_ori_groups.items():
             ori_map[key] = ori_square_to_iter[val]
         fo = order - 1
 
@@ -257,7 +294,7 @@ def get_facet_dof_permutations(n_fp, order):
         raise ValueError('unsupported number of facet points! (%d)' % n_fp)
 
     dof_perms = {}
-    for key, itfun in six.iteritems(ori_map):
+    for key, itfun in ori_map.items():
         dof_perms[key] = [mtx[ii] for ii in itfun(fo)]
 
     dof_perms = dict_to_array(dof_perms)
