@@ -13,14 +13,15 @@ region-local field connectivity.
 """
 import numpy as nm
 
-from sfepy.base.base import assert_
+from sfepy.base.base import assert_, Struct
+from sfepy.discrete.integrals import Integral
 from sfepy.discrete.fem.utils import prepare_remap
 from sfepy.discrete.common.dof_info import expand_nodes_to_dofs
+from sfepy.discrete.common.mappings import get_physical_qps
 from sfepy.discrete.fem.facets import get_facet_dof_permutations
-from sfepy.discrete.fem.fields_base import (FEField, VolumeField, SurfaceField,
-                                            H1Mixin)
+from sfepy.discrete.fem.fields_base import FEField, H1Mixin
 
-class H1NodalMixin(H1Mixin):
+class GlobalNodalLikeBasis(Struct):
 
     def _setup_facet_orientations(self):
         order = self.approx_order
@@ -67,7 +68,7 @@ class H1NodalMixin(H1Mixin):
         facet_desc = nm.array(facet_desc)
         n_dof_per_facet = facet_desc.shape[1]
 
-        cmesh = self.domain.cmesh
+        cmesh = self.cmesh
 
         facets = self.region.entities[dim]
         ii = nm.arange(facets.shape[0], dtype=nm.int32)
@@ -76,7 +77,7 @@ class H1NodalMixin(H1Mixin):
         # Prepare global facet id remapping to field-local numbering.
         remap = prepare_remap(facets, cmesh.num[dim])
 
-        cconn = self.region.domain.cmesh.get_conn(self.region.tdim, dim)
+        cconn = cmesh.get_conn(self.region.tdim, dim)
         offs = cconn.offsets
 
         n_f = self.gel.edges.shape[0] if dim == 1 else self.gel.faces.shape[0]
@@ -124,7 +125,7 @@ class H1NodalMixin(H1Mixin):
         n_dof_per_cell = self.node_desc.bubble.shape[0]
 
         ii = self.region.get_cells()
-        remap = prepare_remap(ii, self.domain.cmesh.n_el)
+        remap = prepare_remap(ii, self.cmesh.n_el)
 
         n_cell = ii.shape[0]
         n_dof = n_dof_per_cell * n_cell
@@ -136,6 +137,31 @@ class H1NodalMixin(H1Mixin):
 
         return n_dof, all_dofs, remap
 
+    def get_surface_basis(self, region):
+        """
+        Get basis for projections to region's facets.
+
+        Notes
+        -----
+        Cannot be uses for all fields because IGA does not support surface
+        mappings.
+        """
+        order = self.approx_order
+
+        integral = Integral('i', order=2*order)
+        geo, mapping = self.get_mapping(region, integral, 'surface')
+        pqps = get_physical_qps(region, integral)
+        qps = pqps.values.reshape(pqps.shape)
+
+        bfs = nm.broadcast_to(
+            geo.bf[..., 0, :],
+            (qps.shape[0], qps.shape[1], geo.bf.shape[3]),
+        )
+
+        return qps, bfs, geo.det[..., 0]
+
+class H1NodalMixin(H1Mixin, GlobalNodalLikeBasis):
+
     def _substitute_dofs(self, subs):
         """
         Perform facet DOF substitutions according to `subs`.
@@ -288,8 +314,8 @@ class H1NodalMixin(H1Mixin):
         coors = fcoors[rffield.econn[0]]
         integral = Integral('i', coors=coors, weights=nm.ones_like(coors[:, 0]))
 
-        rcfield.clear_qp_base()
-        bf = rcfield.get_base('v', False, integral)
+        rcfield.clear_qp_basis()
+        bf = rcfield.eval_basis('v', False, integral)
 
         if gel.name == '2_4':
             fsubs = subs
@@ -311,7 +337,7 @@ class H1NodalMixin(H1Mixin):
 
     def set_dofs(self, fun=0.0, region=None, dpn=None, warn=None):
         """
-        Set the values of DOFs in a given region using a function of space
+        Set the values of DOFs in a given `region` using a function of space
         coordinates or value `fun`.
         """
         if region is None:
@@ -324,21 +350,15 @@ class H1NodalMixin(H1Mixin):
         nods = nm.unique(aux)
 
         if callable(fun):
-            vals = nm.asarray(fun(self.get_coor(nods)))
-
-            try:
-                assert_(vals.size == nods.size * dpn)
-
-            except (TypeError, ValueError):
-                msg = ('wrong function return value shape for setting'
-                       ' DOFs of "%s" field!'
-                       ' (shape %s should be reshapable to %s)'
-                       % (self.name, vals.shape, (nods.size, dpn)))
-
-                raise ValueError(msg)
+            coors = self.get_coor(nods)
+            vals = nm.asarray(fun(coors))
+            if (vals.ndim > 1) and (vals.shape != (len(coors), dpn)):
+                raise ValueError('The projected function return value should be'
+                                 ' (n_point, dpn) == %s, instead of %s!'
+                                 % ((len(coors), dpn), vals.shape))
 
         elif nm.isscalar(fun):
-            vals = nm.repeat([fun], nods.shape[0] * dpn)
+            vals = nm.full(nods.shape[0] * dpn, fun, dtype=nm.dtype(type(fun)))
 
         elif isinstance(fun, nm.ndarray):
             try:
@@ -351,12 +371,12 @@ class H1NodalMixin(H1Mixin):
                        % (self.name, fun.shape, (dpn,)))
                 raise ValueError(msg)
 
-            vals = nm.repeat(fun, nods.shape[0])
+            vals = nm.tile(fun, nods.shape[0])
 
         else:
             raise ValueError('unknown function/value type! (%s)' % type(fun))
 
-        vals.shape = (nods.size, -1)
+        vals.shape = (len(nods), -1)
 
         return nods, vals
 
@@ -377,13 +397,16 @@ class H1NodalMixin(H1Mixin):
 
         return ctx
 
-class H1NodalVolumeField(H1NodalMixin, VolumeField):
+class H1NodalVolumeField(H1NodalMixin, FEField):
+    """
+    Lagrange basis nodal approximation.
+    """
     family_name = 'volume_H1_lagrange'
 
     def interp_v_vals_to_n_vals(self, vec):
         """
         Interpolate a function defined by vertex DOF values using the FE
-        geometry base (P1 or Q1) into the extra nodes, i.e. define the
+        geometry basis (P1 or Q1) into the extra nodes, i.e. define the
         extra DOF values.
         """
         if not self.node_desc.has_extra_nodes():
@@ -395,7 +418,7 @@ class H1NodalVolumeField(H1NodalMixin, VolumeField):
 
             coors = self.poly_space.node_coors
 
-            bf = self.gel.poly_space.eval_base(coors)
+            bf = self.gel.poly_space.eval_basis(coors)
             bf = bf[:,0,:].copy()
 
             conn = self.econn[:, :self.gel.n_vertex]
@@ -406,6 +429,9 @@ class H1NodalVolumeField(H1NodalMixin, VolumeField):
         return enod_vol_val
 
 class H1SNodalVolumeField(H1NodalVolumeField):
+    """
+    Lagrange basis nodal serendipity approximation with order <= 3.
+    """
     family_name = 'volume_H1_serendipity'
 
     def create_basis_context(self):
@@ -418,26 +444,52 @@ class H1SNodalVolumeField(H1NodalVolumeField):
         gps = self.gel.poly_space
         mesh = self.create_mesh(extra_nodes=False)
 
-        ctx = geo_ctx = gps.create_context(mesh.cmesh, 0, 1e-15, 100, 1e-8)
+        ctx = geo_ctx = gps.create_context(self.cmesh, 0, 1e-15, 100, 1e-8)
         ctx.geo_ctx = geo_ctx
 
         return ctx
 
-class H1DiscontinuousField(H1NodalMixin, VolumeField):
+class H1SEMVolumeField(H1NodalVolumeField):
+    """
+    Spectral element method approximation.
+
+    Uses the Lagrange basis with Legendre-Gauss-Lobatto nodes and quadrature.
+    """
+    family_name = 'volume_H1_sem'
+
+    def create_basis_context(self):
+        """
+        Create the context required for evaluating the field basis.
+        """
+        # Hack for tests to pass - the reference coordinates are determined
+        # from vertices only - we can use the Lagrange basis context for the
+        # moment. The true context for Field.evaluate_at() is not implemented.
+        gps = self.gel.poly_space
+        mesh = self.create_mesh(extra_nodes=False)
+
+        ctx = geo_ctx = gps.create_context(self.cmesh, 0, 1e-15, 100, 1e-8)
+        ctx.geo_ctx = geo_ctx
+
+        return ctx
+
+class H1DiscontinuousField(H1NodalMixin, FEField):
+    """
+    The C0 constant-per-cell approximation.
+    """
     family_name = 'volume_H1_lagrange_discontinuous'
 
-    def _setup_global_base(self):
+    def _setup_global_basis(self):
         """
-        Setup global DOF/base function indices and connectivity of the field.
+        Setup global DOF/basis function indices and connectivity of the field.
         """
         self._setup_facet_orientations()
 
         self._init_econn()
 
         ii = self.region.get_cells()
-        self.bubble_remap = prepare_remap(ii, self.domain.cmesh.n_el)
+        self.bubble_remap = prepare_remap(ii, self.cmesh.n_el)
 
-        n_dof = nm.prod(self.econn.shape)
+        n_dof = int(nm.prod(self.econn.shape))
         all_dofs = nm.arange(n_dof, dtype=nm.int32)
         all_dofs.shape = self.econn.shape
 
@@ -485,7 +537,7 @@ class H1DiscontinuousField(H1NodalMixin, VolumeField):
 
         return vertex_dofs
 
-class H1NodalSurfaceField(H1NodalMixin, SurfaceField):
+class H1NodalSurfaceField(H1NodalMixin, FEField):
     """
     A field defined on a surface region.
     """
@@ -494,7 +546,7 @@ class H1NodalSurfaceField(H1NodalMixin, SurfaceField):
     def interp_v_vals_to_n_vals(self, vec):
         """
         Interpolate a function defined by vertex DOF values using the FE
-        surface geometry base (P1 or Q1) into the extra nodes, i.e. define the
+        surface geometry basis (P1 or Q1) into the extra nodes, i.e. define the
         extra DOF values.
         """
         if not self.node_desc.has_extra_nodes():
@@ -508,3 +560,6 @@ class H1NodalSurfaceField(H1NodalMixin, SurfaceField):
 
 class H1SNodalSurfaceField(H1NodalSurfaceField):
     family_name = 'surface_H1_serendipity'
+
+class H1SEMSurfaceField(H1NodalSurfaceField):
+    family_name = 'surface_H1_sem'
