@@ -7,7 +7,7 @@ import numpy as nm
 from sfepy.base.base import (complex_types, dict_from_keys_init,
                              assert_, is_derived_class, ordered_iteritems,
                              insert_static_method, output, get_default,
-                             get_default_attr, Struct, basestr)
+                             get_default_attr, Struct)
 from sfepy.base.ioutils import (skip_read_line, look_ahead_line, read_token,
                                 read_array, pt, enc, dec,
                                 edit_filename,
@@ -15,8 +15,6 @@ from sfepy.base.ioutils import (skip_read_line, look_ahead_line, read_token,
                                 HDF5ContextManager, get_or_create_hdf5_group)
 
 import os.path as op
-import six
-from six.moves import range
 import meshio as meshiolib
 try:
     from meshio import CellBlock as meshio_Cells  # for meshio >= 4.0.3
@@ -137,7 +135,7 @@ def convert_complex_output(out_in):
     real and imaginary parts.
     """
     out = {}
-    for key, val in six.iteritems(out_in):
+    for key, val in out_in.items():
 
         if val.data.dtype in complex_types:
             rval = copy(val)
@@ -216,7 +214,7 @@ class MeshIO(Struct):
         self.set_float_format()
 
     def get_filename_trunk(self):
-        if isinstance(self.filename, basestr):
+        if isinstance(self.filename, str):
             trunk = op.splitext(self.filename)[0]
         else:
             trunk = 'from_descriptor'
@@ -788,7 +786,7 @@ class ComsolMeshIO(MeshIO):
         fd.close()
 
         if out is not None:
-            for key, val in six.iteritems(out):
+            for key, val in out.items():
                 raise NotImplementedError
 
 
@@ -799,10 +797,7 @@ class HDF5MeshIO(MeshIO):
     _all = ''.join(map(chr, list(range(256))))
     _letters = string.ascii_letters + string.digits + '_'
     _rubbish = ''.join([ch for ch in set(_all) - set(_letters)])
-    if sys.version_info[0] >= 3:
-        _tr = str.maketrans(_rubbish, '_' * len(_rubbish))
-    else:
-        _tr = string.maketrans(_rubbish, '_' * len(_rubbish))
+    _tr = str.maketrans(_rubbish, '_' * len(_rubbish))
 
     @staticmethod
     def read_mesh_from_hdf5(filename, group=None, mesh=None):
@@ -909,7 +904,7 @@ class HDF5MeshIO(MeshIO):
             node_sets_groups = fd.create_group(group, 'node_sets',
                                                'node sets groups')
             ii = 0
-            for key, nods in six.iteritems(mesh.nodal_bcs):
+            for key, nods in mesh.nodal_bcs.items():
                 group = fd.create_group(node_sets_groups, 'group%d' % ii,
                                         'node sets group')
                 fd.create_array(group, 'key', enc(key), 'key')
@@ -1154,7 +1149,7 @@ class HDF5MeshIO(MeshIO):
             fd.create_array(ts_group, 'nt', nt, 'normalized time')
 
             name_dict = {}
-            for key, val in six.iteritems(out):
+            for key, val in out.items():
                 if xdmf and mesh.coors.shape[1] == 2:
                     data = expand_data_3d(val.data)
                 else:
@@ -1335,7 +1330,7 @@ class HDF5MeshIO(MeshIO):
             return None
 
         groups = step_group._v_groups
-        for name, data_group in six.iteritems(groups):
+        for name, data_group in groups.items():
             try:
                 key = dec(data_group.dname.read())
 
@@ -1364,7 +1359,7 @@ class HDF5MeshIO(MeshIO):
 
         fd.close()
 
-        for key, val in six.iteritems(th):
+        for key, val in th.items():
             aux = nm.array(val)
             if aux.ndim == 4: # cell data.
                 aux = aux[:,0,:,0]
@@ -1948,7 +1943,7 @@ class ANSYSCDBMeshIO(MeshIO):
                                 hexas, mat_ids_hexas, remap=remap)
 
         mesh.nodal_bcs = {}
-        for key, nods in six.iteritems(nodal_bcs):
+        for key, nods in nodal_bcs.items():
             nods = nods[nods < len(remap)]
             mesh.nodal_bcs[key] = remap[nods]
 
@@ -2419,7 +2414,7 @@ def any_from_filename(filename, prefix_dir=None, file_format=None, mode='r'):
     io : MeshIO subclass instance
         The MeshIO subclass instance corresponding to the kind of `filename`.
     """
-    if not isinstance(filename, basestr):
+    if not isinstance(filename, str):
         if isinstance(filename, MeshIO):
             return filename
 
