@@ -1,10 +1,7 @@
 """
 IO for NURBS and Bezier extraction data.
 """
-from __future__ import absolute_import
 import numpy as nm
-import six
-from six.moves import range
 from sfepy.base.ioutils import HDF5ContextManager, enc, dec
 
 def write_iga_data(filename, group, knots, degrees, control_points, weights,
@@ -60,7 +57,7 @@ def write_iga_data(filename, group, knots, degrees, control_points, weights,
                         'bezier_connectivity')
 
         regs = fd.create_group(group, 'regions', 'regions')
-        for key, val in six.iteritems(regions):
+        for key, val in regions.items():
             fd.create_array(regs, key, val, key)
 
         if name is not None:
