@@ -1,5 +1,4 @@
 """Classes for probing values of Variables, for example, along a line."""
-from __future__ import absolute_import
 import hashlib
 
 from ast import literal_eval
@@ -7,9 +6,8 @@ from ast import literal_eval
 import numpy as nm
 import numpy.linalg as nla
 
-from sfepy.base.base import assert_, basestr, Struct
+from sfepy.base.base import assert_, Struct
 from sfepy.linalg import make_axis_rotation_matrix, norm_l2_along_axis
-import six
 
 def write_results(filename, probe, results):
     """
@@ -25,10 +23,10 @@ def write_results(filename, probe, results):
         The dictionary of probing results. Keys are data names, values are
         the probed values.
     """
-    fd = open(filename, 'w') if isinstance(filename, basestr) else filename
+    fd = open(filename, 'w') if isinstance(filename, str) else filename
 
     fd.write('\n'.join(probe.report()) + '\n')
-    for key, result in six.iteritems(results):
+    for key, result in results.items():
         pars, vals = result
         vals = vals.reshape(pars.shape[0], -1)
         fd.write('\n# %s %d\n' % (key, vals.shape[-1]))
@@ -37,7 +35,7 @@ def write_results(filename, probe, results):
 
         nm.savetxt(fd, aux)
 
-    if isinstance(filename, basestr):
+    if isinstance(filename, str):
         fd.close()
 
 def read_results(filename, only_names=None):
@@ -61,7 +59,7 @@ def read_results(filename, only_names=None):
 
     is_only_names = only_names is not None
 
-    fd = open(filename, 'r') if isinstance(filename, basestr) else filename
+    fd = open(filename, 'r') if isinstance(filename, str) else filename
 
     header = read_header(fd)
     results = {}
