@@ -6,7 +6,7 @@ from functools import partial
 import numpy as nm
 
 from sfepy.base.base import (
-    dict_from_keys_init, select_by_names, is_string, is_integer, is_sequence,
+    dict_from_keys_init, select_by_names, is_sequence,
     output, get_default, Struct, IndexedStruct)
 import sfepy.base.ioutils as io
 from sfepy.base.conf import ProblemConf, get_standard_keywords
@@ -47,12 +47,12 @@ def make_is_save(options):
             self.ilast = 0
             self.save_times = self.save_times0
             if ts is not None:
-                if is_integer(self.save_times0):
+                if isinstance(self.save_times0, int):
                     self.save_times = nm.linspace(ts.t0, ts.t1,
                                                   self.save_times0)
 
         def __call__(self, ts):
-            if is_string(self.save_times) and self.save_times == 'all':
+            if isinstance(self.save_times, str) and self.save_times == 'all':
                 return True
 
             elif isinstance(self.save_times, nm.ndarray):
