@@ -1,4 +1,3 @@
-from __future__ import absolute_import
 
 import numpy as nm
 import numpy.linalg as nla
@@ -11,8 +10,6 @@ from sfepy.linalg import norm_l2_along_axis
 from sfepy.discrete.evaluate import eval_equations
 from sfepy.homogenization.coefs_base import MiniAppBase, CorrMiniApp
 from sfepy.homogenization.utils import coor_to_sym
-import six
-from six.moves import range
 
 def compute_eigenmomenta(em_equation, var_name, problem, eig_vectors,
                          transform=None):
@@ -22,6 +19,7 @@ def compute_eigenmomenta(em_equation, var_name, problem, eig_vectors,
     n_dof, n_eigs = eig_vectors.shape
 
     equations, variables = problem.create_evaluable(em_equation)
+    variables.init_state()
     var = variables[var_name]
 
     n_c = var.n_components
@@ -39,7 +37,7 @@ def compute_eigenmomenta(em_equation, var_name, problem, eig_vectors,
             eigenmomenta[ii, :] = 0.0
 
         else:
-            var.set_data(vec_phi.copy())
+            variables.set_state_parts({var_name: vec_phi})
 
             val = eval_equations(equations, variables)
 
@@ -281,8 +279,8 @@ def detect_band_gaps(mass, freq_info, opts, gap_kind='normal', mtx_b=None):
 
     return slogs, gaps, kinds
 
-def get_callback(mass, method, mtx_b=None, mode='trace'):
-    """
+def get_callback(mass, solver_kind, mtx_b=None, mode='trace'):
+    r"""
     Return callback to solve band gaps or dispersion eigenproblem P.
 
     Notes
@@ -301,21 +299,21 @@ def get_callback(mass, method, mtx_b=None, mode='trace'):
       omega^2 M w = \eta B w"""
 
     def find_zero_callback(f):
-        meigs = eig(mass(f), eigenvectors=False, method=method)
+        meigs = eig(mass(f), eigenvectors=False, solver_kind=solver_kind)
         return meigs
 
     def find_zero_full_callback(f):
         meigs = eig((f**2) * mass(f), mtx_b=mtx_b,
-                    eigenvectors=False, method=method)
+                    eigenvectors=False, solver_kind=solver_kind)
         return meigs
 
     def trace_callback(f):
-        meigs = eig(mass(f), eigenvectors=False, method=method)
+        meigs = eig(mass(f), eigenvectors=False, solver_kind=solver_kind)
         return meigs,
 
     def trace_full_callback(f):
         meigs, mvecs = eig((f**2) * mass(f), mtx_b=mtx_b,
-                           eigenvectors=True, method=method)
+                           eigenvectors=True, solver_kind=solver_kind)
 
         return meigs, mvecs
 
@@ -325,7 +323,7 @@ def get_callback(mass, method, mtx_b=None, mode='trace'):
     return eval(mode + '_callback')
 
 def find_zero(f0, f1, callback, freq_eps, zero_eps, mode):
-    """
+    r"""
     For f \in ]f0, f1[ find frequency f for which either the smallest (`mode` =
     0) or the largest (`mode` = 1) eigenvalue of problem P given by `callback`
     is zero.
@@ -552,7 +550,7 @@ class SimpleEVP(CorrMiniApp):
             mtx_m = mtx_m.toarray()
 
         eigs, mtx_s_phi = eig(mtx_a, mtx_m, return_time=tt,
-                              method=opts.eigensolver)
+                              solver_kind=opts.eigensolver)
         eigs[eigs<0.0] = 0.0
         output('...done in %.2f s' % tt[0])
         output('original eigenfrequencies:')
@@ -593,7 +591,7 @@ class SimpleEVP(CorrMiniApp):
 
         variables = problem.get_variables()
 
-        mtx_phi = nm.empty((variables.di.ptr[-1], mtx_s_phi.shape[1]),
+        mtx_phi = nm.empty((variables.di.n_dof_total, mtx_s_phi.shape[1]),
                            dtype=nm.float64)
 
         make_full = variables.make_full_vec
@@ -608,12 +606,12 @@ class SimpleEVP(CorrMiniApp):
         n_eigs = eigs.shape[0]
 
         out = {}
-        state = problem.create_state()
+        variables = problem.set_default_state()
         for ii in range(n_eigs):
             if (ii >= save[0]) and (ii < (n_eigs - save[1])): continue
-            state.set_full(mtx_phi[:,ii], force=True)
-            aux = state.create_output_dict()
-            for name, val in six.iteritems(aux):
+            variables.set_state(mtx_phi[:,ii], force=True)
+            aux = variables.create_output()
+            for name, val in aux.items():
                 out[name+'%03d' % ii] = val
 
         if self.post_process_hook is not None:
@@ -654,7 +652,7 @@ class SchurEVP(SimpleEVP):
 
         variables = problem.get_variables()
 
-        mtx_phi = nm.empty((variables.di.ptr[-1], mtx_s_phi.shape[1]),
+        mtx_phi = nm.empty((variables.di.n_dof_total, mtx_s_phi.shape[1]),
                            dtype=nm.float64)
 
         make_full = variables.make_full_vec
@@ -665,12 +663,12 @@ class SchurEVP(SimpleEVP):
         eliminated_var = schur['eliminated_var']
 
         mtx_s_phi_schur = - sc.dot(mtx_dib, mtx_s_phi)
-        aux = nm.empty((variables.adi.ptr[-1],), dtype=nm.float64)
-        set = variables.set_state_part
+        aux = nm.empty((variables.adi.n_dof_total,), dtype=nm.float64)
+        setv = variables.set_vec_part
         for ii in range(n_eigs):
-            set(aux, mtx_s_phi[:,ii], primary_var, stripped=True)
-            set(aux, mtx_s_phi_schur[:,ii], eliminated_var,
-                stripped=True)
+            setv(aux, primary_var, mtx_s_phi[:,ii], reduced=True)
+            setv(aux, eliminated_var, mtx_s_phi_schur[:,ii],
+                 reduced=True)
 
             mtx_phi[:,ii] = make_full(aux)
 
@@ -694,7 +692,7 @@ class DensityVolumeInfo(MiniAppBase):
         total_volume = 0.0
         volumes = {}
         densities = {}
-        for region_name, aux in six.iteritems(self.region_to_material):
+        for region_name, aux in self.region_to_material.items():
             vol = vf['volume_' + region_name]
 
             mat_name, item_name = aux
@@ -718,7 +716,7 @@ class DensityVolumeInfo(MiniAppBase):
 
         return Struct(name='density_volume_info',
                       average_density=average_density,
-                      total_volume=total_volume,
+                      total_volume=true_volume,
                       volumes=volumes,
                       densities=densities,
                       to_file_txt=self.to_file_txt)
@@ -732,7 +730,7 @@ class DensityVolumeInfo(MiniAppBase):
         fd.write('average density:\n')
         fd.write(ff % dv_info.average_density)
 
-        for key, val in six.iteritems(dv_info.volumes):
+        for key, val in dv_info.volumes.items():
             fd.write('%s volume:\n' % key)
             fd.write(ff % val)
             fd.write('%s density:\n' % key)
@@ -1230,7 +1228,7 @@ class PhaseVelocity(MiniAppBase):
         mtx_mass = eye * dv_info.average_density
 
         meigs, mvecs = eig(mtx_mass, mtx_b=cat,
-                           eigenvectors=True, method=opts.eigensolver)
+                           eigenvectors=True, solver_kind=opts.eigensolver)
         phase_velocity = 1.0 / nm.sqrt(meigs)
 
         return phase_velocity
