@@ -1,5 +1,3 @@
-from __future__ import print_function
-from __future__ import absolute_import
 import os
 
 import numpy as nm
@@ -10,18 +8,15 @@ from sfepy.base.timing import Timer
 from sfepy.discrete.fem import extend_cell_data, Mesh
 from sfepy.homogenization.utils import coor_to_sym
 from sfepy.base.conf import get_standard_keywords
-from sfepy.discrete import Problem
+from sfepy.discrete import Problem, Region
 from sfepy.base.conf import ProblemConf
 from sfepy.homogenization.coefficients import Coefficients
 from sfepy.homogenization.micmac import get_correctors_from_file_hdf5
 import os.path as op
-import six
-from six.moves import range
+import atexit
 
 shared = Struct()
 
-cc_cache = {}
-
 #
 # TODO : interpolate fvars to macro times. ?mid-points?
 #
@@ -265,7 +260,7 @@ def add_stress_p(out, pb, integral, region, vp, data):
     var = pb.create_variables([vp])[vp]
     var.set_data(data)
 
-    press0 = pb.evaluate('ev_volume_integrate.%s.%s(%s)'
+    press0 = pb.evaluate('ev_integrate.%s.%s(%s)'
                          % (integral, region, vp), verbose=False,
                          mode='el_avg', **{vp: var})
     press = extend_cell_data(press0, pb.domain, region)
@@ -278,7 +273,7 @@ def add_stress_p(out, pb, integral, region, vp, data):
 
 
 def compute_mac_stress_part(pb, integral, region, material, vu, mac_strain):
-    avgmat = pb.evaluate('ev_volume_integrate_mat.%s.%s(%s, %s)'
+    avgmat = pb.evaluate('ev_integrate_mat.%s.%s(%s, %s)'
                          % (integral, region, material, vu), verbose=False,
                          mode='el_avg')
 
@@ -307,7 +302,7 @@ def recover_bones(problem, micro_problem, region, eps0,
     vdp = 'd' + vp
 
     variables = micro_problem.create_variables()
-    to_output = variables.state_to_output
+    to_output = variables.create_output
 
     micro_u, micro_p = variables[vu], variables[vp]
 
@@ -411,7 +406,7 @@ def recover_paraflow(problem, micro_problem, region,
     nodes_y1 = micro_problem.domain.regions['Y1'].vertices
     nodes_y2 = micro_problem.domain.regions['Y2'].vertices
 
-    to_output = micro_problem.variables.state_to_output
+    to_output = micro_problem.variables.create_output
 
     join = os.path.join
     format = get_print_info(problem.domain.mesh.n_el, fill='0')[1]
@@ -480,210 +475,300 @@ def save_recovery_region(mac_pb, rname, filename=None):
     mac_pb.save_state(filename, out=out)
 
 
-def recover_micro_hook(micro_filename, region, macro,
-                       naming_scheme='step_iel',
-                       recovery_file_tag='',
-                       define_args=None, verbose=False):
-    # Create a micro-problem instance.
-    required, other = get_standard_keywords()
-    required.remove('equations')
-    conf = ProblemConf.from_file(micro_filename, required, other,
-                                 verbose=False, define_args=define_args)
-
-    coefs_filename = conf.options.get('coefs_filename', 'coefs')
-    output_dir = conf.options.get('output_dir', '.')
-    coefs_filename = op.join(output_dir, coefs_filename) + '.h5'
-
-    # Coefficients and correctors
-    coefs = Coefficients.from_file_hdf5(coefs_filename)
-    corrs = get_correctors_from_file_hdf5(dump_names=coefs.save_names)
-
-    recovery_hook = conf.options.get('recovery_hook', None)
-
-    if recovery_hook is not None:
-        recovery_hook = conf.get_function(recovery_hook)
-        pb = Problem.from_conf(conf, init_equations=False, init_solvers=False)
-
-        format = get_print_info(pb.domain.mesh.n_el, fill='0')[1]
-
-        output('recovering microsctructures...')
-        timer = Timer(start=True)
-        output_fun = output.output_function
-        output_level = output.level
-        for ii, iel in enumerate(region.cells):
-            output.level = output_level
-            output('micro: %d (el=%d)' % (ii, iel))
-
-            local_macro = {}
-            for k, v in six.iteritems(macro):
-                local_macro[k] = v[ii, 0]
-
-            output.set_output(quiet=not(verbose))
-            out = recovery_hook(pb, corrs, local_macro)
-            output.output_function = output_fun
-
-            if ii == 0:
-                new_keys = []
-                new_data = {}
-                new_idxs = []
-                for k in six.iterkeys(local_macro):
-                    if k not in macro:
-                        new_keys.append(k)
-                        new_data[k] = []
-
-            new_idxs.append(ii)
-            for jj in new_keys:
-                new_data[jj].append(local_macro[jj])
-
-            # save data
-            if out is not None:
-                suffix = format % iel
-                micro_name = pb.get_output_name(extra='recovered_'
-                                                + recovery_file_tag + suffix)
-                filename = op.join(output_dir, op.basename(micro_name))
-                fpv = pb.conf.options.get('file_per_var', False)
-                pb.save_state(filename, out=out,
-                              file_per_var=fpv)
-
-        output('...done in %.2f s' % timer.stop())
-
-        for jj in new_keys:
-            lout = new_data[jj]
-            macro[jj] = nm.zeros((nm.max(new_idxs) + 1, 1) + lout[0].shape,
-                                 dtype=lout[0].dtype)
-            out = macro[jj]
-            for kk, ii in enumerate(new_idxs):
-                out[ii, 0] = lout[kk]
+_recovery_global_dict = {}
 
 
-def recover_micro_hook_eps(micro_filename, region,
-                           eval_var, nodal_values, const_values, eps0,
-                           recovery_file_tag='',
-                           define_args=None, verbose=False, use_cache=False):
-    # Create a micro-problem instance.
-    required, other = get_standard_keywords()
-    required.remove('equations')
-    conf = ProblemConf.from_file(micro_filename, required, other,
-                                 verbose=False, define_args=define_args)
+def destroy_pool():
+    if 'pool' in _recovery_global_dict:
+        _recovery_global_dict['pool'].close()
 
-    coefs_filename = conf.options.get('coefs_filename', 'coefs')
-    output_dir = conf.options.get('output_dir', '.')
-    coefs_filename = op.join(output_dir, coefs_filename) + '.h5'
 
-    if not use_cache:
-        cc_cache.clear()
- 
-    # Coefficients and correctors
-    if coefs_filename not in cc_cache:
-        coefs = Coefficients.from_file_hdf5(coefs_filename)
-        corrs = get_correctors_from_file_hdf5(dump_names=coefs.save_names)
-        cc_cache[coefs_filename] = coefs, corrs
+atexit.register(destroy_pool)
+
+
+def _recovery_hook(args):
+    idx, label, local_macro, verbose = args
+    pb, corrs, recovery_hook = _recovery_global_dict['hook_args']
+
+    output.set_output(quiet=False)
+    output.level = label[1]
+    output(label[0])
+    output.set_output(quiet=not(verbose))
+
+    if isinstance(corrs, list):
+        corrs = corrs[idx]
+
+    return recovery_hook(pb, corrs, local_macro)
+
+
+def get_recovery_points(region, eps0):
+    rcoors = region.domain.mesh.coors[region.get_entities(0), :]
+    rcmin = nm.min(rcoors, axis=0)
+    rcmax = nm.max(rcoors, axis=0)
+    nn = nm.round((rcmax - rcmin) / eps0)
+    if nm.prod(nn) == 0:
+        raise ValueError(
+            'incompatible recovery region and microstructure size!')
+
+    cs = []
+    for ii, n in enumerate(nn):
+        cs.append(nm.arange(n) * eps0 + rcmin[ii])
+
+    ccoors = nm.empty((int(nm.prod(nn)), nn.shape[0]), dtype=nm.float64)
+    for ii, icoor in enumerate(nm.meshgrid(*cs, indexing='ij')):
+        ccoors[:, ii] = icoor.flatten()
+
+    return ccoors
+
+
+def recover_micro_hook(micro_filename, region, macro, eps0,
+                       region_mode='el_centers', eval_mode='constant',
+                       eval_vars=None, corrs=None, recovery_file_tag='',
+                       define_args=None, output_dir=None, verbose=False):
+    """
+    Parameters
+    ----------
+    micro_filename : str
+        The definition file of the microproblem.
+    region : Region or array
+        The macroscopic region to be recovered. If array, the centers of
+        microscopic RVEs (Representative Volume Element) are assumed to be
+        stored in it. If Region, the RVE centers are computed according to
+        `region_mode`, see below.
+    macro : dict of arrays or tuples
+        Either macroscopic values (if array) or the tuple
+        (mode, eval_var, nodal_values) is expected. The tuple is used to
+        evaluate the macroscopic values in given points of RVEs (see
+        'eval_mode`). `mode` can be 'val', 'grad', 'div', or 'cauchy_strain'.
+    eps0 : float
+        The size of the microstructures (RVE).
+    region_mode : {'el_centers', 'tiled'}
+        If 'el_centers', the RVE centers are identical to the element centers
+        of the macroscopic FE mesh. If 'tiled', the recovered region is tiled
+        by rescaled RVEs.
+    eval_mode : {'constant', 'continuous'}
+        If 'constant', the macroscopic fields are evaluated only at the RVE
+        centers. If 'continuous', the fields are evaluated at all points of
+        the RVE mesh.
+    eval_vars : list of variables
+        The list of variables use to evaluate the macroscopic fields.
+    corrs : dict of CorrSolution
+        The correctors for recovery.
+    recovery_file_tag : str
+        The tag which is appended to the output file.
+    define_args : dict
+        The define arguments for the microscopic problem.
+    output_dir : str
+        The output directory.
+    verbose : bool
+        The verbose terminal output.
+    """
+    import sfepy.base.multiproc_proc as multi
+
+    if 'micro_problem' not in _recovery_global_dict:
+        # Create a micro-problem instance.
+        required, other = get_standard_keywords()
+        required.remove('equations')
+        conf = ProblemConf.from_file(micro_filename, required, other,
+                                     verbose=False, define_args=define_args)
+        if output_dir is not None:
+            conf.options.output_dir = output_dir
+
+        recovery_hook = conf.options.get('recovery_hook', None)
+        pb = None
+        if recovery_hook is not None:
+            recovery_hook = conf.get_function(recovery_hook)
+
+            if corrs is None:
+                # Coefficients and correctors
+                coefs_filename = conf.options.get('coefs_filename', 'coefs')
+                coefs_filename = op.join(conf.options.get('output_dir', '.'),
+                                         coefs_filename + '.h5')
+                coefs = Coefficients.from_file_hdf5(coefs_filename)
+                corrs = \
+                    get_correctors_from_file_hdf5(dump_names=coefs.save_names)
+
+            pb = Problem.from_conf(conf, init_equations=False,
+                                   init_solvers=False)
+
+        _recovery_global_dict['micro_problem'] = pb, corrs, recovery_hook
     else:
-        coefs, corrs = cc_cache[coefs_filename]
+        pb, corrs, recovery_hook = _recovery_global_dict['micro_problem']
 
-    recovery_hook = conf.options.get('recovery_hook', None)
+    is_multiproc = pb.conf.options.get('multiprocessing', True)\
+        and multi.use_multiprocessing
 
     if recovery_hook is not None:
-        recovery_hook = conf.get_function(recovery_hook)
-        pb = Problem.from_conf(conf, init_equations=False, init_solvers=False)
+        if eval_mode not in ['constant', 'continuous']:
+            raise ValueError(f"Evaluation mode '{eval_mode}' not implemented!")
 
-        # Get tiling of a given region
-        rcoors = region.domain.mesh.coors[region.get_entities(0), :]
-        rcmin = nm.min(rcoors, axis=0)
-        rcmax = nm.max(rcoors, axis=0)
-        nn = nm.round((rcmax - rcmin) / eps0)
-        if nm.prod(nn) == 0:
-            output('inconsistency in recovery region and microstructure size!')
-            return
+        if isinstance(region, Region):
+            if region_mode == 'el_centers':
+                dim = region.domain.mesh.dim
+                ccoors = region.cmesh.get_centroids(dim)
+                ccoors = ccoors[region.get_cells(), :]
+                recovery_ids = region.get_entities(-1)
+                recovery_ids_s = [f'(element {k})' for k in recovery_ids]
+            elif region_mode == 'tiled':
+                ccoors = get_recovery_points(region, eps0)
+                recovery_ids = nm.arange(ccoors.shape[0])
+                recovery_ids_s = [f'(point {k})' for k in recovery_ids]
+            else:
+                raise ValueError(
+                    f"Region mode '{region_mode}' not implemented!")
 
-        cs = []
-        for ii, n in enumerate(nn):
-            cs.append(nm.arange(n) * eps0 + rcmin[ii])
+        else:
+            ccoors = region
+            recovery_ids = nm.arange(ccoors.shape[0])
+            recovery_ids_s = [f'(point {k})' for k in recovery_ids]
 
-        x0 = nm.empty((int(nm.prod(nn)), nn.shape[0]), dtype=nm.float64)
-        for ii, icoor in enumerate(nm.meshgrid(*cs, indexing='ij')):
-            x0[:, ii] = icoor.flatten()
+        nrec = ccoors.shape[0]
 
-        mesh = pb.domain.mesh
-
-        coors, conn, outs, ndoffset = [], [], [], 0
-        # Recover region
-        mic_coors = (mesh.coors - mesh.get_bounding_box()[0, :]) * eps0
-        evfield = eval_var.field
-
-        output('recovering microsctructures...')
+        output(f'recovering {nrec} microsctructures...')
         timer = Timer(start=True)
-        output_fun = output.output_function
-        output_level = output.level
+        output_level, output_fun = output.level, output.output_function
 
-        for ii, c0 in enumerate(x0):
-            local_macro = {'eps0': eps0}
-            local_coors = mic_coors + c0
-            # Inside recovery region?
-            v = nm.ones((evfield.region.entities[0].shape[0], 1))
-            v[evfield.vertex_remap[region.entities[0]]] = 0
-            no = nm.sum(v)
-            aux = evfield.evaluate_at(local_coors, v)
-            if no > 0 and (nm.sum(aux) / no) > 1e-3:
-                continue
+        _recovery_global_dict['hook_args'] = pb, corrs, recovery_hook
 
-            output.level = output_level
-            output('micro: %d / %d' % (ii, x0.shape[0]))
+        if is_multiproc:
+            multi_local_macro = []
+            num_workers = nm.min([multi.cpu_count(), nrec])
+            _recovery_global_dict['pool'] = multi.Pool(processes=num_workers)
+        else:
+            outs = []
 
-            for k, v in six.iteritems(nodal_values):
-                local_macro[k] = evfield.evaluate_at(local_coors, v)
-            for k, v in six.iteritems(const_values):
-                local_macro[k] = v
+        # Recover micro. region
+        mesh = pb.domain.mesh
+        bbox = mesh.get_bounding_box()
+        mic_coors = (mesh.coors - 0.5 * (bbox[1, :] + bbox[0, :])) * eps0
+        coors, conn, ndoffset, rec_ids = [], [], 0, []
 
-            output.set_output(quiet=not(verbose))
-            outs.append(recovery_hook(pb, corrs, local_macro))
-            output.output_function = output_fun
+        if eval_vars is not None and eval_mode == 'constant':
+            new_macro = {}
+            for k, v in macro.items():
+                if isinstance(v, tuple):
+                    emode, evar, val = v
+                    efield = eval_vars[evar].field
+                    new_macro[k] = efield.evaluate_at(ccoors, val, mode=emode)
+                else:
+                    new_macro[k] = v
+
+            macro = new_macro
+
+        for ii in range(nrec):
+            local_coors = mic_coors.copy()
+            local_coors[:, :ccoors.shape[1]] += ccoors[ii]
             coors.append(local_coors)
             conn.append(mesh.get_conn(mesh.descs[0]) + ndoffset)
             ndoffset += mesh.n_nod
+            rec_ids.append(nm.ones((mesh.n_el,)) * recovery_ids[ii])
 
-    output('...done in %.2f s' % timer.stop())
+            label = (f'micro: {ii + 1}/{nrec} {recovery_ids_s[ii]}',
+                     output_level)
 
-    # Collect output variables
-    outvars = {}
-    for k, v in six.iteritems(outs[0]):
-        if v.var_name in outvars:
-            outvars[v.var_name].append(k)
-        else:
-            outvars[v.var_name] = [k]
+            local_macro = {'eps0': eps0}
 
-    # Split output by variables/regions
-    pvs = pb.create_variables(outvars.keys())
-    outregs = {k: pvs[k].field.region.get_entities(-1) for k in outvars.keys()}
-    nrve = len(coors)
-    coors = nm.vstack(coors)
-    ngroups = nm.tile(mesh.cmesh.vertex_groups.squeeze(), (nrve,))
-    conn = nm.vstack(conn)
-    cgroups = nm.tile(mesh.cmesh.cell_groups.squeeze(), (nrve,))
+            if eval_vars is not None and eval_mode == 'continuous':
+                for k, v in macro.items():
+                    if isinstance(v, tuple):
+                        emode, evar, val = v
+                        efield = eval_vars[evar].field
 
-    # Get region mesh and data
-    for k, cidxs in six.iteritems(outregs):
-        gcidxs = nm.hstack([cidxs + mesh.n_el * ii for ii in range(nrve)])
-        rconn = conn[gcidxs]
-        remap = -nm.ones((coors.shape[0],), dtype=nm.int32)
-        remap[rconn] = 1
-        vidxs = nm.where(remap > 0)[0]
-        remap[vidxs] = nm.arange(len(vidxs))
-        rconn = remap[rconn]
-        rcoors = coors[vidxs, :]
+                        # # Inside recovery region?
+                        # from sfepy.base.base import debug; debug()
+                        # v = nm.ones((efield.region.entities[0].shape[0], 1))
+                        # v[evfield.vertex_remap[region.entities[0]]] = 0
+                        # no = nm.sum(v)
+                        # aux = efield.evaluate_at(local_coors, v)
+                        # if no > 0 and (nm.sum(aux) / no) > 1e-3:
+                        #     print('not inside!')
+                        #     continue
 
-        out = {}
-        for ifield in outvars[k]:
-            data = [outs[ii][ifield].data for ii in range(nrve)]
-            out[ifield] = Struct(name='output_data',
-                                 mode=outs[0][ifield].mode,
-                                 dofs=None,
-                                 var_name=k,
-                                 data=nm.vstack(data))
+                        local_macro[k] = efield.evaluate_at(local_coors, val,
+                                                            mode=emode)
+                    else:
+                        local_macro[k] = v
+            else:
+                for k, v in macro.items():
+                    if k.startswith('_'):
+                        local_macro[k] = v
+                    else:
+                        local_macro[k] = v[ii, ...]
 
-        micro_name = pb.get_output_name(extra='recovered%s_%s'
-                                        % (recovery_file_tag, k))
-        filename = op.join(output_dir, op.basename(micro_name))
-        mesh_out = Mesh.from_data('recovery_%s' % k, rcoors, ngroups[vidxs],
-                                  [rconn], [cgroups[gcidxs]], [mesh.descs[0]])
-        mesh_out.write(filename, io='auto', out=out)
+            if is_multiproc:
+                multi_local_macro.append((ii, label, local_macro, verbose))
+            else:
+                outs.append(_recovery_hook((ii, label, local_macro, verbose)))
+
+        rec_ids = nm.hstack(rec_ids).reshape((-1, 1, 1, 1))
+
+        if is_multiproc:
+            pool = _recovery_global_dict['pool']
+            outs = list(pool.map(_recovery_hook, multi_local_macro))
+
+        output.level, output.output_function = output_level, output_fun
+
+        # Collect output data - by region
+        outregs_data = {}
+        outregs_info = {None: ('ALL', nm.arange(pb.domain.mesh.n_el))}
+        vn_reg = {None: None}
+        for k, v in outs[0].items():
+            if hasattr(v, 'region_name'):
+                rn = v.region_name
+                if rn not in outregs_info:
+                    reg = pb.domain.regions[rn]
+                    outregs_info[rn] = (rn, reg.get_entities(-1))
+            else:
+                vn = getattr(v, 'var_name', None)
+                if vn not in vn_reg:
+                    reg = pb.create_variables(vn)[vn].field.region
+                    rn = reg.name
+                    vn_reg[vn] = rn
+                    if rn not in outregs_info:
+                        outregs_info[rn] = (rn, reg.get_entities(-1))
+                else:
+                    rn = vn_reg[vn]
+
+            if rn in outregs_data:
+                outregs_data[rn].append(k)
+            else:
+                outregs_data[rn] = [k]
+
+        nrve = len(coors)
+        coors = nm.vstack(coors)
+        ngroups = nm.tile(mesh.cmesh.vertex_groups.squeeze(), (nrve,))
+        conn = nm.vstack(conn)
+        cgroups = nm.tile(mesh.cmesh.cell_groups.squeeze(), (nrve,))
+        # Get region mesh and data
+        output_dir = pb.conf.options.get('output_dir', '.')
+        for rn in outregs_data.keys():
+            rlabel, cidxs = outregs_info[rn]
+            gcidxs = nm.hstack([cidxs + mesh.n_el * ii for ii in range(nrve)])
+            rconn = conn[gcidxs]
+            remap = -nm.ones((coors.shape[0],), dtype=nm.int32)
+            remap[rconn] = 1
+            vidxs = nm.where(remap > 0)[0]
+            remap[vidxs] = nm.arange(len(vidxs))
+            rconn = remap[rconn]
+            rcoors = coors[vidxs, :]
+
+            out = {}
+            for ivar in outregs_data[rn]:
+                data = [outs[ii][ivar].data for ii in range(nrve)]
+                out[ivar] = Struct(name='output_data',
+                                   mode=outs[0][ivar].mode,
+                                   data=nm.vstack(data))
+
+            out['recovery_id'] = Struct(name='output_data',
+                                        mode='cell',
+                                        data=rec_ids[gcidxs, ...])
+            micro_name = pb.get_output_name(extra='recovered_%s%s'
+                                            % (rlabel, recovery_file_tag))
+            filename = op.join(output_dir, op.basename(micro_name))
+            mesh_out = Mesh.from_data('recovery_%s' % rlabel, rcoors,
+                                      ngroups[vidxs], [rconn],
+                                      [cgroups[gcidxs]], [mesh.descs[0]])
+            mesh_out.write(filename, io='auto', out=out)
+            output(f'output saved to "{filename}"')
+
+        output('...done in %.2f s' % timer.stop())
