@@ -1,13 +1,11 @@
-from __future__ import absolute_import
 import numpy as nm
 
 from sfepy.base.base import assert_
 from sfepy.linalg import norm_l2_along_axis as norm
 from sfepy.linalg import dot_sequences, insert_strided_axis
 from sfepy.discrete import PolySpace
-from sfepy.discrete.fem.mappings import VolumeMapping
+from sfepy.discrete.fem.mappings import FEMapping
 from sfepy.mechanics.tensors import dim2sym
-from six.moves import range
 
 def create_transformation_matrix(coors):
     """
@@ -108,7 +106,7 @@ def create_mapping(coors, gel, order):
 
     Returns
     -------
-    mapping : VolumeMapping instance
+    mapping : FEMapping instance
         The reference element face mapping.
     """
     # Strip 'z' component (should be 0 now...).
@@ -121,7 +119,7 @@ def create_mapping(coors, gel, order):
     seq_conn = nm.arange(seq_coors.shape[0], dtype=nm.int32)
     seq_conn.shape = sh[:2]
 
-    mapping = VolumeMapping(seq_coors, seq_conn, gel=gel, order=1)
+    mapping = FEMapping(seq_coors, seq_conn, gel=gel, order=1)
 
     return mapping
 
@@ -148,7 +146,7 @@ def describe_geometry(field, region, integral):
     """
     # Coordinates of element vertices.
     sg, _ = field.get_mapping(region, integral, 'surface')
-    sd = field.surface_data[region.name]
+    sd = field.extra_data[f'sd_{region.name}']
     coors = field.coors[sd.econn[:, :sg.n_ep]]
 
     # Coordinate transformation matrix (transposed!).
@@ -163,8 +161,8 @@ def describe_geometry(field, region, integral):
 
     qp = integral.get_qp(gel.name)
     ps = PolySpace.any_from_args(None, gel, field.approx_order)
-    membrane_geo = vm.get_mapping(qp[0], qp[1], poly_space=ps)
-    membrane_geo.bf[:] = ps.eval_base(qp[0])
+    membrane_geo = vm.get_mapping(qp[0], qp[1], ps.eval_basis(qp[0]),
+                                  poly_space=ps)
 
     return mtx_t, membrane_geo
 
