@@ -1,9 +1,6 @@
 """
 Some utilities for work with units of physical quantities.
 """
-from __future__ import absolute_import
-import six
-from six.moves import range
 try:
     import sympy as sm
 except ImportError:
@@ -108,7 +105,7 @@ class Unit(Struct):
         if omit is None:
             omit = num_prefixes
 
-        values = [val for key, val in six.iteritems(prefixes) if key not in omit]
+        values = [val for key, val in prefixes.items() if key not in omit]
         coefs = nm.array(values, dtype=nm.float64)
         coefs.sort()
         ii = nm.searchsorted(coefs, bias*coef, side='left')
@@ -206,7 +203,7 @@ class Quantity(Struct):
         self.def_coef = float(self.symbolic_value.subs(self.def_names))
 
         coef_dict = {}
-        for key, val in six.iteritems(self.def_units):
+        for key, val in self.def_units.items():
             coef_dict[val.name] = self.units[key].coef
         self.coef_dict = coef_dict
 
