@@ -1,7 +1,4 @@
-from __future__ import print_function
-from __future__ import absolute_import
 import sys
-from six.moves import range
 sys.path.append('.')
 
 import numpy as nm
