@@ -1,10 +1,6 @@
-from __future__ import print_function
-from __future__ import absolute_import
 import numpy as nm
-import six
-from six.moves import range
 
-class geometry(object):
+class geometry:
     """The geometry is given by a sets of points (d0), lines (d1), surfaces
     (d2) and volumes (d3). A lines are constructed from 2 points, a surface from
     any number of lines, a volume from any number of surfaces.
@@ -83,15 +79,15 @@ class geometry(object):
         print("  dimension:", self.dim)
         print("  points:", len(self.d0))
         if verbose:
-            for k, v in six.iteritems(self.d0):
+            for k, v in self.d0.items():
                 print("    %d - %s" % (k, v.getstr()))
         print("  lines:", len(self.d1))
         if verbose:
-            for k, v in six.iteritems(self.d1):
+            for k, v in self.d1.items():
                 print("    %d - " % k, v.points)
         print("  surfaces:", len(self.d2))
         if verbose:
-            for k, v in six.iteritems(self.d2):
+            for k, v in self.d2.items():
                 if v.is_hole:
                     aux = '(hole)'
                 else:
@@ -99,7 +95,7 @@ class geometry(object):
                 print("    %d%s - " % (k, aux), v.lines)
         print("  volumes:", len(self.d3))
         if verbose:
-            for k, v in six.iteritems(self.d3):
+            for k, v in self.d3.items():
                 print("    %d - " % k, v.surfaces)
         print("Physical entities:")
         if self.dim == 2:
@@ -169,7 +165,7 @@ class geometry(object):
                 self.addline(lid, [points[ii], points[ii + 1]])
                 lines.append(lid)
 
-            for s in six.itervalues(self.d2):
+            for s in self.d2.values():
                 try:
                     idx = s.lines.index(l.n)
                 except ValueError:
@@ -436,7 +432,7 @@ class geometry(object):
 
         return geo
 
-class geomobject(object):
+class geomobject:
     def getn(self):
         return self.n
 
