@@ -1,12 +1,9 @@
 """
 Functions for a high-level PETSc-based parallelization.
 """
-from __future__ import absolute_import
-import time
 import os
 
 import numpy as nm
-from six.moves import range
 
 def init_petsc_args():
     try:
@@ -20,10 +17,20 @@ def init_petsc_args():
 
 init_petsc_args()
 
-from petsc4py import PETSc
-from mpi4py import MPI
+try:
+    from petsc4py import PETSc
+
+except (ModuleNotFoundError, ImportError):
+    PETSc = None
+
+try:
+    from mpi4py import MPI
+
+except (ModuleNotFoundError, ImportError):
+    MPI = None
 
 from sfepy.base.base import assert_, output, ordered_iteritems, Struct
+from sfepy.base.timing import Timer
 from sfepy.discrete.common.region import Region
 from sfepy.discrete.fem.fe_surface import FESurface
 
@@ -33,7 +40,7 @@ def partition_mesh(mesh, n_parts, use_metis=True, verbose=False):
     available.
     """
     output('partitioning mesh into %d subdomains...' % n_parts, verbose=verbose)
-    tt = time.clock()
+    timer = Timer(start=True)
 
     if use_metis:
         try:
@@ -54,7 +61,7 @@ def partition_mesh(mesh, n_parts, use_metis=True, verbose=False):
 
     else:
         ii = nm.arange(n_parts)
-        n_cell_parts = mesh.n_el / n_parts + ((mesh.n_el % n_parts) > ii)
+        n_cell_parts = mesh.n_el // n_parts + ((mesh.n_el % n_parts) > ii)
         output('cell counts:', n_cell_parts, verbose=verbose)
         assert_(sum(n_cell_parts) == mesh.n_el)
         assert_(nm.all(n_cell_parts > 0))
@@ -62,7 +69,7 @@ def partition_mesh(mesh, n_parts, use_metis=True, verbose=False):
         offs = nm.cumsum(nm.r_[0, n_cell_parts])
         cell_tasks = nm.digitize(nm.arange(offs[-1]), offs) - 1
 
-    output('...done in', time.clock() - tt, verbose=verbose)
+    output('...done in', timer.stop(), verbose=verbose)
 
     return cell_tasks
 
@@ -296,7 +303,7 @@ def verify_task_dof_maps(dof_maps, id_map, field, use_expand_dofs=False,
 
     Returns the vector with a task number for each DOF.
     """
-    tt = time.clock()
+    timer = Timer(start=True)
     if verbose:
         output('verifying...')
         output('total number of DOFs:', field.n_nod)
@@ -352,7 +359,7 @@ def verify_task_dof_maps(dof_maps, id_map, field, use_expand_dofs=False,
     assert_(n_dof == len(id_map))
     assert_(n_dof == len(nm.unique(id_map)))
 
-    output('...done in', time.clock() - tt, verbose=verbose)
+    output('...done in', timer.stop(), verbose=verbose)
 
     return vec
 
@@ -405,7 +412,9 @@ def distribute_field_dofs(field, gfd, use_expand_dofs=False,
             global_dofs = expand_dofs(global_dofs, field.n_components)
 
         dof_map = dof_maps[0]
-        petsc_dofs_range = (gfd.coffsets[0], gfd.coffsets[0] + dof_map[3])
+
+        petsc_dofs_range = (int(gfd.coffsets[0]),
+                            int(gfd.coffsets[0]) + dof_map[3])
         petsc_dofs_conn = id_map[global_dofs]
 
     else:
@@ -535,7 +544,7 @@ def get_sizes(petsc_dofs_range, n_dof, n_components):
     """
     Get (local, total) sizes of a vector and local equation range.
     """
-    drange = tuple(n_components * nm.asarray(petsc_dofs_range))
+    drange = tuple([n_components * ii for ii in petsc_dofs_range])
     n_loc = drange[1] - drange[0]
     n_all_dof = n_dof * n_components
     sizes = (n_loc, n_all_dof)
@@ -607,20 +616,29 @@ def expand_dofs(dofs, n_components):
 
     return edofs
 
-def view_petsc_local(data, name='data', viewer=None, comm=None):
+def call_in_rank_order(fun, comm=None):
     """
-    View local PETSc `data` called `name`. The data object has to have
-    `.view()` method.
+    Call a function `fun` task by task in the task rank order.
     """
     if comm is None:
         comm = PETSc.COMM_WORLD
 
     for rank in range(comm.size):
         if rank == comm.rank:
-            output('contents of local %s on rank %d:' % (name, rank))
-            data.view(viewer=viewer)
+            fun(rank, comm)
         comm.barrier()
 
+def view_petsc_local(data, name='data', viewer=None, comm=None):
+    """
+    View local PETSc `data` called `name`. The data object has to have
+    `.view()` method.
+    """
+    def _view(rank, comm):
+        output('contents of local %s on rank %d:' % (name, rank))
+        data.view(viewer=viewer)
+
+    call_in_rank_order(_view, comm=comm)
+
 def create_local_petsc_vector(pdofs):
     """
     Create a local PETSc vector with the size corresponding to `pdofs`.
@@ -751,29 +769,30 @@ def assemble_rhs_to_petsc(prhs, rhs, pdofs, drange, is_overlap=True,
     if comm is None:
         comm = PETSc.COMM_WORLD
 
+    timer = Timer()
     if is_overlap:
         output('setting rhs values...', verbose=verbose)
-        tt = time.clock()
+        timer.start()
         rdofs = nm.where((pdofs < drange[0]) | (pdofs >= drange[1]), -1, pdofs)
         prhs.setOption(prhs.Option.IGNORE_NEGATIVE_INDICES, True)
         prhs.setValues(rdofs, rhs, PETSc.InsertMode.INSERT_VALUES)
-        output('...done in', time.clock() - tt, verbose=verbose)
+        output('...done in', timer.stop(), verbose=verbose)
 
         output('assembling rhs...', verbose=verbose)
-        tt = time.clock()
+        timer.start()
         prhs.assemble()
-        output('...done in', time.clock() - tt, verbose=verbose)
+        output('...done in', timer.stop(), verbose=verbose)
 
     else:
         output('setting rhs values...', verbose=verbose)
-        tt = time.clock()
+        timer.start()
         prhs.setValues(pdofs, rhs, PETSc.InsertMode.ADD_VALUES)
-        output('...done in', time.clock() - tt, verbose=verbose)
+        output('...done in', timer.stop(), verbose=verbose)
 
         output('assembling rhs...', verbose=verbose)
-        tt = time.clock()
+        timer.start()
         prhs.assemble()
-        output('...done in', time.clock() - tt, verbose=verbose)
+        output('...done in', timer.stop(), verbose=verbose)
 
 def assemble_mtx_to_petsc(pmtx, mtx, pdofs, drange, is_overlap=True,
                           comm=None, verbose=False):
@@ -783,11 +802,13 @@ def assemble_mtx_to_petsc(pmtx, mtx, pdofs, drange, is_overlap=True,
     if comm is None:
         comm = PETSc.COMM_WORLD
 
+    timer = Timer()
+
     lgmap = PETSc.LGMap().create(pdofs, comm=comm)
     pmtx.setLGMap(lgmap, lgmap)
     if is_overlap:
         output('setting matrix values...', verbose=verbose)
-        tt = time.clock()
+        timer.start()
         mask = (pdofs < drange[0]) | (pdofs >= drange[1])
         nnz_per_row = nm.diff(mtx.indptr)
         mtx2 = mtx.copy()
@@ -795,22 +816,22 @@ def assemble_mtx_to_petsc(pmtx, mtx, pdofs, drange, is_overlap=True,
         mtx2.eliminate_zeros()
         pmtx.setValuesLocalCSR(mtx2.indptr, mtx2.indices, mtx2.data,
                                PETSc.InsertMode.INSERT_VALUES)
-        output('...done in', time.clock() - tt, verbose=verbose)
+        output('...done in', timer.stop(), verbose=verbose)
 
         output('assembling matrix...', verbose=verbose)
-        tt = time.clock()
+        timer.start()
         pmtx.assemble()
-        output('...done in', time.clock() - tt, verbose=verbose)
+        output('...done in', timer.stop(), verbose=verbose)
 
 
     else:
         output('setting matrix values...', verbose=verbose)
-        tt = time.clock()
+        timer.start()
         pmtx.setValuesLocalCSR(mtx.indptr, mtx.indices, mtx.data,
                                PETSc.InsertMode.ADD_VALUES)
-        output('...done in', time.clock() - tt, verbose=verbose)
+        output('...done in', timer.stop(), verbose=verbose)
 
         output('assembling matrix...', verbose=verbose)
-        tt = time.clock()
+        timer.start()
         pmtx.assemble()
-        output('...done in', time.clock() - tt, verbose=verbose)
+        output('...done in', timer.stop(), verbose=verbose)
