@@ -6,15 +6,12 @@ The standard geometry elements can be plotted by running::
 
   $ python sfepy/postprocess/plot_facets.py
 """
-from __future__ import absolute_import
 import numpy as nm
 import matplotlib.pyplot as plt
 
 from sfepy.linalg import (get_perpendiculars, normalize_vectors,
                           make_axis_rotation_matrix)
 from sfepy.postprocess.plot_dofs import _get_axes, plot_mesh, plot_global_dofs
-import six
-from six.moves import range
 
 def plot_geometry(ax, gel):
     """
@@ -32,6 +29,8 @@ def plot_edges(ax, gel, length):
     dim = gel.dim
     ax = _get_axes(ax, dim)
 
+    if gel.edges is None: return ax
+
     l2 = 0.5 * length
     for ii, edge in enumerate(gel.edges):
         cc = gel.coors[edge]
@@ -144,7 +143,7 @@ if __name__ == '__main__':
     from sfepy.discrete.fem.geometry_element import (GeometryElement,
                                                      geometry_data)
 
-    for key, gd in six.iteritems(geometry_data):
+    for key, gd in geometry_data.items():
         if key == '1_2' : continue
 
         gel = GeometryElement(key)
