@@ -1,4 +1,3 @@
-from __future__ import absolute_import
 import os
 import sfepy
 from sfepy.base.base import load_classes, insert_static_method
@@ -13,8 +12,8 @@ solver_files = [name for name in solver_files
 solver_table = load_classes(solver_files,
                             [AutoFallbackSolver,
                              LinearSolver, NonlinearSolver,
-                             TimeSteppingSolver, EigenvalueSolver,
-                             QuadraticEVPSolver,
+                             TimeStepController, TimeSteppingSolver,
+                             EigenvalueSolver, QuadraticEVPSolver,
                              OptimizationSolver], package_name='sfepy.solvers')
 
 
