@@ -1,4 +1,3 @@
-from __future__ import absolute_import
 
 import numpy as nm
 import numpy.linalg as nla
@@ -10,8 +9,6 @@ from sfepy.solvers.solvers import OptimizationSolver
 
 import scipy.optimize as sopt
 import scipy.optimize.linesearch as linesearch
-import six
-from six.moves import range
 
 def conv_test(conf, it, of, of0, ofg_norm=None):
     """
@@ -269,7 +266,7 @@ class FMinSteepestDescent(OptimizationSolver):
             else:
                 ofg = ofg1.copy()
 
-            for key, val in six.iteritems(time_stats):
+            for key, val in time_stats.items():
                 if len(val):
                     output('%10s: %7.2f [s]' % (key, val[-1]))
 
