@@ -1,14 +1,12 @@
-from __future__ import absolute_import
-import time
 
 import numpy as nm
 import numpy.linalg as nla
 
 from sfepy.base.base import output, get_default, Struct
 from sfepy.base.log import Log, get_logging_conf
-from sfepy.solvers.solvers import SolverMeta, NonlinearSolver
+from sfepy.base.timing import Timer
+from sfepy.solvers.solvers import NonlinearSolver
 from .nls import conv_test
-import six
 
 class StabilizationFunction(Struct):
     """
@@ -40,8 +38,8 @@ class StabilizationFunction(Struct):
         # Indices to the state vector.
         ii = {}
         ii['u'] = variables.get_indx(ns['u'])
-        ii['us'] = variables.get_indx(ns['u'], stripped=True)
-        ii['ps'] = variables.get_indx(ns['p'], stripped=True)
+        ii['us'] = variables.get_indx(ns['u'], reduced=True)
+        ii['ps'] = variables.get_indx(ns['p'], reduced=True)
         self.indices = ii
 
         materials = problem.get_materials()
@@ -96,7 +94,7 @@ class StabilizationFunction(Struct):
             region = term.region
             vg, _ = field.get_mapping(region, term.integral, 'volume')
             cells = region.get_cells()
-            d2 = problem.domain.get_element_diameters(cells, vg, dm)
+            d2 = problem.domain.get_element_diameters(cells, vg.volume, dm)
 
             self.diameters2 = d2
 
@@ -144,8 +142,6 @@ class Oseen(NonlinearSolver):
     """
     name = 'nls.oseen'
 
-    __metaclass__ = SolverMeta
-
     _parameters = [
         ('stabil_mat', 'str', None, True,
          'The name of stabilization material.'),
@@ -210,13 +206,13 @@ class Oseen(NonlinearSolver):
         problem = get_default(problem, conf.problem,
                               '`problem` parameter needs to be set!')
 
+        timer = Timer()
         time_stats = {}
 
         stabil = problem.get_materials()[conf.stabil_mat]
         ns, ii = stabil.function.function.get_maps()
 
         variables = problem.get_variables()
-        update_var = variables.set_data_from_state
         make_full_vec = variables.make_full_vec
 
         output('problem size:')
@@ -232,9 +228,11 @@ class Oseen(NonlinearSolver):
 
         err0 = -1.0
         it = 0
+        ls_n_iter = 0
+        ls_status = {}
         while 1:
             vec_x_prev_f = make_full_vec(vec_x_prev)
-            update_var(ns['b'], vec_x_prev_f, ns['u'])
+            variables[ns['b']].set_data(vec_x_prev_f, variables[ns['u']].indx)
 
             vec_b = vec_x_prev_f[ii['u']]
             b_norm = nla.norm(vec_b, nm.inf)
@@ -259,14 +257,14 @@ class Oseen(NonlinearSolver):
             else:
                 adimensionalize = False
 
-            tt = time.clock()
+            timer.start()
             try:
                 vec_r = fun(vec_x)
             except ValueError:
                 ok = False
             else:
                 ok = True
-            time_stats['residual'] = time.clock() - tt
+            time_stats['residual'] = timer.stop()
             if ok:
                 err = nla.norm(vec_r)
                 if it == 0:
@@ -291,20 +289,21 @@ class Oseen(NonlinearSolver):
                 ## mat.viscosity = viscosity / b_norm
                 ## vec_r[indx_us] /= b_norm
 
-            tt = time.clock()
+            timer.start()
             try:
                 mtx_a = fun_grad(vec_x)
             except ValueError:
                 ok = False
             else:
                 ok = True
-            time_stats['matrix'] = time.clock() - tt
+            time_stats['matrix'] = timer.stop()
             if not ok:
                 raise RuntimeError('giving up...')
 
-            tt = time.clock()
-            vec_dx = lin_solver(vec_r, x0=vec_x, mtx=mtx_a)
-            time_stats['solve'] = time.clock() - tt
+            timer.start()
+            vec_dx = lin_solver(vec_r, x0=vec_x, mtx=mtx_a, status=ls_status)
+            time_stats['solve'] = timer.stop()
+            ls_n_iter += ls_status['n_iter']
 
             vec_e = mtx_a * vec_dx - vec_r
             lerr = nla.norm(vec_e)
@@ -318,7 +317,7 @@ class Oseen(NonlinearSolver):
             dx_norm = nla.norm(vec_dx)
             output('||dx||: %.2e' % dx_norm)
 
-            for kv in six.iteritems(time_stats):
+            for kv in time_stats.items():
                 output('%10s: %7.2f [s]' % kv)
 
             vec_x_prev = vec_x.copy()
@@ -369,6 +368,9 @@ class Oseen(NonlinearSolver):
             status['err'] = err
             status['err_ns'] = err_ns
             status['condition'] = condition
+            status['n_iter'] = it
+            status['ls_n_iter'] = -1
+            status['time'] = sum(v for v in time_stats.values())
 
         if conf.log.plot is not None:
             if self.log is not None:
