@@ -1,21 +1,18 @@
-from __future__ import absolute_import
-import time
 
 import numpy as nm
 import numpy.linalg as nla
 import scipy.sparse as sp
 
 from sfepy.base.base import output, get_default, debug
-from sfepy.solvers.solvers import SolverMeta
+from sfepy.base.timing import Timer
 from sfepy.solvers.nls import Newton, conv_test
 from sfepy.linalg import compose_sparse
-import six
-from six.moves import range
 
 class SemismoothNewton(Newton):
     r"""
-    The semi-smooth Newton method for solving problems of the following
-    structure:
+    The semi-smooth Newton method.
+
+    This method is suitable for solving problems of the following structure:
 
     .. math::
         \begin{split}
@@ -35,8 +32,6 @@ class SemismoothNewton(Newton):
     """
     name = 'nls.semismooth_newton'
 
-    __metaclass__ = SolverMeta
-
     _parameters = [
         ('semismooth', 'bool', True, False,
          """If True, use the semi-smooth algorithm. Otherwise a non-smooth
@@ -56,8 +51,9 @@ class SemismoothNewton(Newton):
         ('ls_on', 'float', 0.99999, False,
          """Start the backtracking line-search by reducing the step, if
             :math:`||f(x^i)|| / ||f(x^{i-1})||` is larger than `ls_on`."""),
-        ('ls_red', '0.0 < float < 1.0', 0.1, False,
-         'The step reduction factor in case of correct residual assembling.'),
+        ('ls_red', 'dict', {'regular' : 0.1, 'steepest_descent' : 0.01}, False,
+         """The step reduction factor in case of correct residual assembling
+            for regular and steepest descent modes."""),
         ('ls_red_warp', '0.0 < float < 1.0', 0.001, False,
          """The step reduction factor in case of failed residual assembling
             (e.g. the "warp violation" error caused by a negative volume
@@ -84,6 +80,7 @@ class SemismoothNewton(Newton):
         lin_solver = get_default(lin_solver, self.lin_solver)
         status = get_default(status, self.status)
 
+        timer = Timer()
         time_stats = {}
 
         vec_x = vec_x0.copy()
@@ -105,7 +102,7 @@ class SemismoothNewton(Newton):
             vec_dx0 = vec_dx;
             i_ls = 0
             while 1:
-                tt = time.clock()
+                timer.start()
 
                 try:
                     vec_smooth_r = fun_smooth(vec_x)
@@ -136,7 +133,7 @@ class SemismoothNewton(Newton):
 
                     ok = True
 
-                time_stats['residual'] = time.clock() - tt
+                time_stats['residual'] = timer.stop()
 
                 if ok:
                     vec_r = nm.r_[vec_smooth_r, vec_semismooth_r]
@@ -211,7 +208,7 @@ class SemismoothNewton(Newton):
             if condition >= 0:
                 break
 
-            tt = time.clock()
+            timer.start()
 
             if not reuse_matrix:
                 mtx_jac = self.compute_jacobian(vec_x, fun_smooth_grad,
@@ -222,9 +219,9 @@ class SemismoothNewton(Newton):
             else:
                 reuse_matrix = False
 
-            time_stats['matrix'] = time.clock() - tt
+            time_stats['matrix'] = timer.stop()
 
-            tt = time.clock()
+            timer.start()
 
             if step_mode == 'regular':
                 vec_dx = lin_solver(vec_r, mtx=mtx_jac)
@@ -241,9 +238,9 @@ class SemismoothNewton(Newton):
             else:
                 vec_dx = mtx_jac.T * vec_r
 
-            time_stats['solve'] = time.clock() - tt
+            time_stats['solve'] = timer.stop()
 
-            for kv in six.iteritems(time_stats):
+            for kv in time_stats.items():
                 output('%10s: %7.2f [s]' % kv)
 
             vec_x -= vec_dx
