@@ -11,15 +11,16 @@ BEGIN_C_DECLS
 #include "fmfield.h"
 #include "refmaps.h"
 
-int32 dw_lin_elastic_iso( FMField *out, FMField *strain,
-			  FMField *lam, FMField *mu, Mapping *vg,
-			  int32 isDiff );
 int32 dw_lin_elastic( FMField *out, float64 coef, FMField *strain,
 		      FMField *mtxD, Mapping *vg,
 		      int32 isDiff );
 int32 d_lin_elastic( FMField *out, float64 coef, FMField *strainV,
 		     FMField *strainU, FMField *mtxD, Mapping *vg );
 
+int32 d_sd_lin_elastic(FMField *out, float64 coef, FMField *gradV,
+                       FMField *gradU, FMField *gradW, FMField *mtxD,
+                       Mapping *vg);
+
 int32 dw_lin_prestress( FMField *out, FMField *stress, Mapping *vg );
 
 int32 dw_lin_strain_fib( FMField *out, FMField *mtxD, FMField *mat,
@@ -34,6 +35,8 @@ int32 dq_cauchy_strain( FMField *out, FMField *state, int32 offset,
 			Mapping *vg,
 			int32 *conn, int32 nEl, int32 nEP );
 
+int32 dw_nonsym_elastic(FMField *out, FMField *grad, FMField *mtxD,
+                        Mapping *vg, int32 isDiff);
 
 END_C_DECLS
 
