@@ -1,8 +1,7 @@
-from __future__ import absolute_import
 import numpy as nm
 
 from sfepy.base.base import assert_, Struct
-from sfepy.terms.terms import terms
+from sfepy.terms.terms import Term, terms
 from sfepy.terms.terms_hyperelastic_base import\
     HyperElasticBase, HyperElasticFamilyData
 
@@ -91,8 +90,9 @@ class GenYeohTLTerm(HyperElasticTLBase):
         return out
 
     def stress_function(self, out, mat, *fargs, **kwargs):
-        coef, exp = mat[:, :, :, :1], mat[:, :, :, 1:]
         det_f, tr_c, inv_c = fargs
+        mat = HyperElasticBase.tile_mat(mat, det_f.shape[0])
+        coef, exp = mat[:, :, :, :1], mat[:, :, :, 1:]
 
         i_3 = det_f**2
         ident = nm.array([1., 1, 1, 0, 0, 0])
@@ -147,13 +147,14 @@ class GenYeohTLTerm(HyperElasticTLBase):
         return tan_mod
 
     def tan_mod_function(self, out, mat, *fargs, **kwargs):
-        coef, exp = mat[:, :, :, :1], mat[:, :, :, 1:]
         det_f, tr_c, inv_c = fargs
+        mat = HyperElasticBase.tile_mat(mat, det_f.shape[0])
+        coef, exp = mat[:, :, :, :1], mat[:, :, :, 1:]
 
         n_cells, n_qps, _, _ = out.shape
         for cell in range(n_cells):
             for qp in range(n_qps):
-                _inv_c = inv_c[cell, qp]
+                _inv_c = inv_c[cell, qp, :, 0]
                 _c_inv = nm.array([
                     [_inv_c[0], _inv_c[3], _inv_c[4]],
                     [_inv_c[3], _inv_c[1], _inv_c[5]],
@@ -251,8 +252,9 @@ class OgdenTLTerm(HyperElasticTLBase):
         return out
 
     def stress_function(self, out, mat, *fargs, **kwargs):
-        coef, exp = mat[:, :, :, :1], mat[:, :, :, 1:]
         det_f, sym_c, _, _ = fargs
+        mat = HyperElasticBase.tile_mat(mat, det_f.shape[0])
+        coef, exp = mat[:, :, :, :1], mat[:, :, :, 1:]
 
         # compute principal stretches and directions
         c_mats = sym_c[:, :, [[0, 3, 4], [3, 1, 5], [4, 5, 2]], 0]
@@ -329,8 +331,9 @@ class OgdenTLTerm(HyperElasticTLBase):
         return tan_mod
 
     def tan_mod_function(self, out, mat, *fargs, **kwargs):
-        coef, exp = mat[:, :, :, :1], mat[:, :, :, 1:]
         det_f, sym_c, tr_c, inv_c = fargs
+        mat = HyperElasticBase.tile_mat(mat, det_f.shape[0])
+        coef, exp = mat[:, :, :, :1], mat[:, :, :, 1:]
 
         # compute principal stretches and directions
         c_mats = sym_c[:, :, [[0, 3, 4], [3, 1, 5], [4, 5, 2]], 0]
@@ -443,6 +446,7 @@ class BulkPressureTLTerm(HyperElasticTLBase):
     """
     name = 'dw_tl_bulk_pressure'
     arg_types = ('virtual', 'state', 'state_p')
+    arg_geometry_types = {('state_p', None) : {'facet_extra' : 'facet'}}
     arg_shapes = {'virtual' : ('D', 'state'), 'state' : 'D', 'state_p' : 1}
     family_data_names = ['det_f', 'sym_inv_c']
 
@@ -557,6 +561,7 @@ class VolumeTLTerm(HyperElasticTLBase):
     """
     name = 'dw_tl_volume'
     arg_types = ('virtual', 'state')
+    arg_geometry_types = {('virtual', None) : {'facet_extra' : 'facet'}}
     arg_shapes = {'virtual' : (1, None), 'state' : 'D'}
     family_data_names = ['mtx_f', 'det_f', 'sym_inv_c']
 
@@ -680,20 +685,24 @@ class HyperElasticSurfaceTLFamilyData(HyperElasticFamilyData):
     cache_name = 'tl_surface_common'
     data_names = ('mtx_f', 'det_f', 'inv_f')
 
-    def __call__(self, state, region, integral, integration,
+    def __call__(self, state, region, integral, geometry_type,
                  step=0, derivative=None):
         sg, _ = state.field.get_mapping(region,
-                                        integral, integration,
+                                        integral, geometry_type[0],
                                         get_saved=True)
-        sd = state.field.surface_data[region.name]
+        sd = state.field.extra_data[f'sd_{region.name}']
 
         vec = state(step=step, derivative=derivative)
 
-        st_shape = state.get_data_shape(integral, integration, region.name)
+        st_shape = state.get_data_shape(integral, geometry_type[0],
+                                        region.name)
         data = self.init_data_struct(st_shape, name='surface_family_data')
 
         fargs = tuple([getattr(data, k) for k in self.data_names])
         fargs = fargs + (vec, sg, sd.fis, state.field.econn)
+        fargs = Term.translate_fargs_mapping(self.family_function,
+                                             list(fargs))
+
         self.family_function(*fargs)
 
         return data
@@ -720,12 +729,12 @@ class SurfaceFluxTLTerm(HyperElasticSurfaceTLBase):
         - parameter_1 : :math:`p`
         - parameter_2 : :math:`\ul{u}^{(n-1)}`
     """
-    name = 'd_tl_surface_flux'
+    name = 'ev_tl_surface_flux'
     arg_types = ('material_1', 'material_2', 'parameter_1', 'parameter_2')
     arg_shapes = {'material_1' : 'D, D', 'material_2' : '1, 1',
                   'parameter_1' : 1, 'parameter_2' : 'D'}
     family_data_names = ['det_f', 'inv_f']
-    integration = 'surface_extra'
+    integration = 'facet_extra'
 
     function = staticmethod(terms.d_tl_surface_flux)
 
@@ -775,15 +784,15 @@ class SurfaceTractionTLTerm(HyperElasticSurfaceTLBase):
                    'state' : 'D'},
                   {'opt_material' : None}]
     family_data_names = ['det_f', 'inv_f']
-    integration = 'surface_extra'
+    integration = 'facet_extra'
 
     function = staticmethod(terms.dw_tl_surface_traction)
 
     def get_fargs(self, mat, virtual, state,
                   mode=None, term_mode=None, diff_var=None, **kwargs):
         sg, _ = self.get_mapping(virtual)
-        sd = virtual.field.surface_data[self.region.name]
-        bf = virtual.field.get_base(sd.bkey, 0, self.integral)
+        sd = virtual.field.extra_data[f'sd_{self.region.name}']
+        bf = virtual.field.eval_basis(sd.bkey, 0, self.integral)
 
         name = state.name
         fd = self.get_family_data(state, self.region, self.integral,
@@ -820,19 +829,19 @@ class VolumeSurfaceTLTerm(HyperElasticSurfaceTLBase):
     :Arguments:
         - parameter : :math:`\ul{u}`
     """
-    name = 'd_tl_volume_surface'
+    name = 'ev_tl_volume_surface'
     arg_types = ('parameter',)
     arg_shapes = {'parameter' : 'D'}
     family_data_names = ['det_f', 'inv_f']
-    integration = 'surface_extra'
+    integration = 'facet_extra'
 
     function = staticmethod(terms.d_tl_volume_surface)
 
     def get_fargs(self, parameter,
                   mode=None, term_mode=None, diff_var=None, **kwargs):
         sg, _ = self.get_mapping(parameter)
-        sd = parameter.field.surface_data[self.region.name]
-        bf = parameter.field.get_base(sd.bkey, 0, self.integral)
+        sd = parameter.field.extra_data[f'sd_{self.region.name}']
+        bf = parameter.field.eval_basis(sd.bkey, 0, self.integral)
 
         name = parameter.name
         fd = self.get_family_data(parameter, self.region, self.integral,
